/*********************************************************************
 * Desktop Station Core Library for Arduino
 *
 * Copyright (C) 2014 Yaasan
 * Not use for commercial use
 */

#ifndef DSCORELIB_H
#define DSCORELIB_H


#include <Arduino.h>



#define DSGATEWAY_VERSION 0x03
//#define DEBUG false
#define SIZE_PACKET 8
#define SIZE_TXBUF	16
#define TIME_REPLY 200

#define SPEEDSTEP_DCC28 0
#define SPEEDSTEP_DCC14 1
#define SPEEDSTEP_DCC127 2
#define SPEEDSTEP_MM14 0
#define SPEEDSTEP_MM28 1


#define		GO_FWD					0x01
#define		GO_REV					0x02

#define		DSC_STATE_IDLE				0
#define		DSC_STATE_POWERON			1





/* 特殊・拡張機能用 */
#define EXCMD_NONE		0


/* Defined functions */
class DSCoreLib
{
  private:
	
	word stacked_bit_num;
	uint8_t flag_send_twice;
	uint8_t len_send_twice;
	uint8_t index_send;
	
	uint8_t state_cvscan;
	uint8_t count_cvscan_state;
	
	word address_dsc_cvread;
	
	word number_dsc_cvwrite;
	word value_dsc_cvwrite;
	
	word cvval_max[16];
	word cvval_min[16];
	word offset_adc_current;

	word len_last_sendTx;
	word len_last_sendBits;
	uint32_t data_end32_last_send;	//最後に送信した末尾パケット32bitデータ
	word gTxBufNum;
	uint32_t gTxBuf[SIZE_TXBUF];
	
	uint8_t MAIN_scanList(DS_ITEM *inList, uint8_t inScanIndex);
	uint8_t Main_scanFunction(DS_ITEM *inList, uint8_t inScanIndex);
	uint8_t Main_generateDCCFncPacket(DS_ITEM *inList, uint8_t inScanIndex, uint8_t inSendTwice);
	uint8_t Main_decodeLocDirection(DS_ITEM *inList, word address, uint8_t direction, uint8_t *outType, uint8_t *outPackets, uint8_t *outLen);
	void Main_decodeLocFunction(DS_ITEM *inList, word address, uint8_t function, uint8_t power, uint8_t *outType, uint8_t *outPackets, uint8_t *outLen, uint64_t *ioFuncBuf);
	void Main_decodeAccessory(word address, uint8_t power, uint8_t *outType, uint8_t *outPackets, uint8_t *outLen);
	void Main_decodeLocSpeed(DS_ITEM *inList, word address, short speed, uint8_t *outType, uint8_t *outPackets, uint8_t *outLen, uint8_t inSpeedStep);
	void Main_decodeAddress( word address, uint8_t *outpType, uint8_t *outpAddr1, uint8_t *outpAddr2, uint8_t *outpLen);
	bool GetCVValueFromMap(uint8_t *inCntData, uint8_t *outpValue, uint8_t *outpHighByte, uint8_t *outpLowByte);
	void getCVData(word inAddress, word *inMaxData, word *inMinData, uint8_t inD, uint8_t inK, uint8_t inCntReset, uint8_t inCntCV, word inAdcOffset);
	
	//新DSCore

	void pushPacketBit(uint8_t aBit);
	void pushPacketByte(uint8_t aByte);

	void sendPackets(uint8_t *inPackets, uint8_t inLen, uint8_t inPreambleCnt);
	void sendInitialPacket(uint8_t inPreambleCnt);

	
  public:
	DSCoreLib();
	//~DSCoreLib();
	
	word gThresholdCurrent;
	uint8_t gRetryCount;
	uint8_t gCutOut;
	


	
	void Init();
	void InitPins();
	uint8_t Scan();
	void Clear();
	void SendReset();

	//DCCパケット送信用スタック関連の初期化・再生成関数
	void ClearPacketStack();
	void PreparePacketStack();
	
	//Command API
	boolean IsPower();
	boolean SetPower(boolean power);
	boolean SetPowerEx(boolean power);
	boolean SetLocoSpeed(word address, int inSpeed);
	boolean SetLocoSpeedEx(word address, int inSpeed, int inProtcol);
	boolean SetLocoFunction(word address, unsigned char inFunction, unsigned char inPower);
	boolean SetLocoFunctionRaw(word address, uint64_t inFunctions);
	boolean SetLocoDirection(word address, unsigned char inDirection);
	boolean SetTurnout(word address, uint8_t inSwitch);
	boolean WriteConfig_Ops(word address, word number, uint8_t value);
	boolean WriteConfig_Dir( word number, uint8_t value);
	boolean ReadConfig( word number, uint8_t *value);
	boolean ReadConfigEx( word number, uint8_t *value);
	
	void FillPacketBit();

	word GetLocIDProtocol(uint8_t address);
	word IsScanDataLen();
	word IsScanDataSent();
	uint32_t ScanData();
	bool CheckTwiceSend();
	boolean CVReadPostProcess(uint8_t *value);
	word GetCVReadValue();
	word GetCurrent(word inAvg);
	word GetCurrentAvg();
};

#endif
