
#include <arduino.h>
#include "DSCoreM_Type.h"
#include "DSCoreM_Common.h"

uint8_t gMode = MODE_POWEROFF;
unsigned int gPrevTime = 0;

void DS_GateOff(void)
{
	gMode = MODE_POWEROFF;
	DS_OutputOff();

}

void DS_OutputOff(void)
{
	gpio_put(PIN_DCC_A, 0);
	gpio_put(PIN_DCC_B, 0);
}

void DS_OutputBrake(void)
{
	gpio_put(PIN_DCC_A, 1);
	gpio_put(PIN_DCC_B, 1);
}


void DS_Power(uint8_t inMode)
{
	if( inMode == 1)
	{
		/* パワーオン */
		gMode = MODE_POWERON;
		
		//電流を流す
		DS_WriteBit(0);	
		
	}
	else
	{
		DS_GateOff();	
	
	}

}

uint8_t DS_IsPowerOn(void)
{
	return gMode == MODE_POWERON ? 1 : 0;
}


void DS_WriteBit(uint8_t inBit)
{
	
	if(gMode == MODE_POWEROFF)
	{
		return;
	}

	if( inBit == 0)
	{
		/* PORT B 1,2 */
		gpio_put(PIN_DCC_A, 1);
		gpio_put(PIN_DCC_B, 0);
	}
	else if (inBit == 1)
	{
		/* PORT B 1,2 */
		gpio_put(PIN_DCC_A, 0);
		gpio_put(PIN_DCC_B, 1);
	}
	else
	{
		DS_GateOff();	
	}
}

void DS_WaitSec(uint8_t inSec)
{
	int i;
	
	for( i = 0; i < 5 * inSec; i++)
	{
		delay(200);
	}
}


uint8_t DS_CalculateCRC(uint8_t *inPackets, uint8_t inLen)
{
	int i = 1;
	uint8_t aCRC = inPackets[0];

	for( i = 1; i < inLen; i++)
	{
		aCRC = aCRC ^ inPackets[i];
	}
	
	return aCRC;

}

uint8_t DS_CalcSpeedStep(short inSpeed, uint8_t inDiv)
{
	
	if( inSpeed == 0)
	{
		return 0;
	}
	else
	{
		
		short aSpeed = ((inSpeed >> 2) * inDiv) >> 8;
		
		return (uint8_t)aSpeed + 1;
	}
	
}


uint8_t DS_GetLocIDProtocol(uint8_t address)
{
	if( address < 0x04)
	{
		return PROTOCOL_MM2;
	}
	else if( (address >= 0x30) && (address <= 0x33))
	{
		return PROTOCOL_MM2_ACC;
	}
	else if( (address >= 0x38) && (address <= 0x3F))
	{
		return PROTOCOL_DCC_ACC;
	}
	else if( (address >= 0x40) && (address <= 0x70))
	{
		return PROTOCOL_MFX;
	}
	else if( (address >= 0xC0) && (address <= 0xFF))
	{
		return PROTOCOL_DCC;
	}
	else
	{
		return PROTCOL_UNKNOWN;
	}
}


uint8_t DS_ABSdiff(uint8_t inUpper, uint8_t inLower)
{
	if( inUpper > inLower)
	{
		return inUpper - inLower;
	}
	else
	{
		return inLower - inUpper;
	}
}



void DS_WaitClk(uint8_t inTime4us)
{
	
	unsigned long aCurrent = micros();
	
	while(1)
	{
		if( micros() >= aCurrent)
		{
			if((micros() - aCurrent) >= inTime4us)
			{
				break;
			}
		}
		else
		{
			if((micros() + (0xFFFFFFFF - aCurrent)) >= inTime4us)
			{
				break;
			}
		}
		
	}
	
	
}


void DS_InitWaitClkX(void)
{

}

void DS_WaitClkX(uint8_t inWaitTime)
{
	
	switch(inWaitTime)
	{
		
	case 110:
		delayMicroseconds(58);
		break;
	case 210:
	default:
		delayMicroseconds(105);
		break;
	}

}

