/*
  OzOLED.h - 0.96' I2C 128x64 OLED Driver Library
  2014 Copyright (c) OscarLiang.net  All right reserved.
 
  Author: Oscar Liang
  
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

*/

#ifndef OzOLED_data_H
#define OzOLED_data_H

#include <Arduino.h>

#define OzOLED_Max_X					64	//128 Pixels
#define OzOLED_Max_Y					128	//64  Pixels

#define OLED_ADDRESS					0x3C
#define I2C_400KHZ						1	// 0 to use default 100Khz, 1 for 400Khz

// registers
#define OzOLED_COMMAND_MODE				0x80
#define OzOLED_DATA_MODE				0x40

// commands
#define OzOLED_CMD_DISPLAY_OFF			0xAE
#define OzOLED_CMD_DISPLAY_ON			0xAF
#define OzOLED_CMD_NORMAL_DISPLAY		0xA6
#define OzOLED_CMD_INVERSE_DISPLAY		0xA7
#define OzOLED_CMD_SET_BRIGHTNESS		0x81

#define OzOLED_RIGHT_SCROLL				0x26
#define OzOLED_LEFT_SCROLL				0x27
#define OzOLED_SET_VERTICAL_SCROLL_AREA 0xA3
#define OzOLED_VERTICAL_RIGHT_SCROLL	0x29
#define OzOLED_VERTICAL_LEFT_SCROLL		0x2A
#define OzOLED_CMD_ACTIVATE_SCROLL		0x2F
#define OzOLED_CMD_DEACTIVATE_SCROLL	0x2E

#define HORIZONTAL_ADDRESSING	0x00
#define VERTICAL_ADDRESSING		0x01
#define PAGE_ADDRESSING			0x02

#define Scroll_Left				0x00
#define Scroll_Right			0x01
#define Scroll_Up				0x02
#define Scroll_Down				0x03

#define Scroll_2Frames			0x07
#define Scroll_3Frames			0x04
#define Scroll_4Frames			0x05
#define Scroll_5Frames			0x00
#define Scroll_25Frames			0x06
#define Scroll_64Frames			0x01
#define Scroll_128Frames		0x02
#define Scroll_256Frames		0x03

#define ICON_NONE		0
#define ICON_THUNDER	1
#define ICON_RIGHT		2
#define ICON_LEFT		3
#define ICON_INFO		4
#define ICON_SQUARE		5
#define ICON_ERR		6
#define ICON_OFF		7
#define ICON_HOME		8


typedef void (*VoidFunction) ();


class OzOLED {

public:

	uint8_t addressingMode;

	void sendCommand(uint8_t command);
	void sendCommand(uint8_t command, uint8_t command2);
	void sendData(uint8_t Data);

	void clearLine(uint8_t Y);

	void drawIcon(byte X, byte Y, byte No, boolean inNot);
	void drawIcon2(byte X, byte Y, byte No, byte shift, boolean inNot);

	void DrawQRCodeURL(uint8_t inX, uint8_t inY);
	void DrawLogo(byte inY);
	void DrawFunctionIcon(uint8_t funcno, uint8_t x, uint8_t y, bool inNot);
	void DrawWordWithCircle(const char *str, uint8_t X, uint8_t Y);

	void drawLineHorizonal(uint8_t Y, uint8_t data=255);
	void drawBigNumber(uint8_t X, uint8_t Y, uint8_t No);

	void printChar(uint16_t c, uint8_t X=255, uint8_t Y=255);
	void printChar2(uint16_t C, uint16_t C2, uint8_t X=255, uint8_t Y=255);
	void printCharM(uint16_t c, uint8_t X=255, uint8_t Y=255);
	void printCharL(uint16_t c, uint8_t X=255, uint8_t Y=255);
	void printCharNOT(uint16_t c, uint8_t X=255, uint8_t Y=255);
	void printCharNOT2(uint16_t C, uint16_t C2, uint8_t X=255, uint8_t Y=255);
	void printCharNOT3(uint16_t C, uint8_t X=255, uint8_t Y=255);
	void printString(const char *str, uint8_t X=255, uint8_t Y=255, uint8_t numChar=255);
	void printStringM(char *str, uint8_t X=255, uint8_t Y=255, uint8_t numChar=255);
	void printStringL(char *str, uint8_t X=255, uint8_t Y=255, uint8_t numChar=255);

	void printStringNOT(const char *str, uint8_t X=255, uint8_t Y=255, uint8_t numChar=255);
	
	
	uint8_t printNumber(long n, uint8_t X=255, uint8_t Y=255);
	uint8_t printNumber(float float_num, uint8_t prec, uint8_t X, uint8_t Y);
	void printBigNumber(const char *number, uint8_t column=0, uint8_t page=0, uint8_t numChar=255); 
	void drawBitmap(const uint8_t *bitmaparray, uint8_t X, uint8_t Y, uint8_t width, uint8_t height);
	void drawBitmap2(const uint8_t *bitmaparray, uint8_t X, uint8_t Y, uint8_t width, uint8_t height);
	
	void drawSpeedBar(uint8_t X, uint8_t Y, word inSpeed, boolean inNot);
	void drawSpeedBarW16(uint8_t X, uint8_t Y, word inSpeed, boolean inNot);

	void drawTOLine(uint8_t X, uint8_t Y, uint8_t inType, boolean inNot);
	void drawTOLine2(uint8_t X, uint8_t Y, uint8_t inBits, uint8_t inType, boolean inNot);

	void drawBigNumberX2(uint8_t X, uint8_t Y, uint8_t No, uint8_t Negative=0);
	void printBigNumberX2(const char *number, uint8_t column=0, uint8_t page=0, uint8_t numChar=255); 
	void printBigNumberX2NOT(const char *number, uint8_t column=0, uint8_t page=0, uint8_t numChar=255); 
	void init();
	
	void setCursorXY(uint8_t Column, uint8_t Row);
	void clearDisplay();
	void lcd_clear();
	
	void setNormalDisplay();
	void setInverseDisplay();
	void setPowerOff();
	void setPowerOn();
	void setPageMode();
	void setBrightness(uint8_t Brightness);
	

};

extern OzOLED OzOled;  // OzOLED object 

#endif


