//////////////////////////////////////////////////////////////////////////////////
// 
// Desktop Station ShareMem Manager
// Copyright(C)'2023- DesktopStation
// 
// Not use for commercial use
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//


#ifndef SHAREMEMMANAGER_LIB_H
#define SHAREMEMMANAGER_LIB_H

#include <Arduino.h>


#define FIRMWARE_VER '6' /*0-9, a-z, A-Z*/ 

#define SHAREMEM_STATUS_SIZE 264	//32 + 16 + 32x2 + 19 *8 = 112 + 144=264
#define SHAREMEM_SIZE 48
#define SHAREMEM_ACCSIZE 32

#define MAX_LOCSTATUS_WEB 8

#define SMEM_POWER		0
#define SMEM_ERROR		1
#define SMEM_EDC		2
#define SMEM_CUR		3
#define SMEM_SEQ		4
#define SMEM_S88_1		5
#define SMEM_S88_2		6
#define SMEM_S88_3		7
#define SMEM_S88_4		8

typedef struct
{
	word mAddress;	//Marklin Address Format
	uint8_t mSpeed;	//(1/4 Data)
	uint8_t mDir;
	word mFuncBuf1;
	word mFuncBuf2;
} DATA_LOC;



/* Defined functions */

class ShareMemManager
{
	private:
	
		uint8_t buffer_sharedStatusMem[SHAREMEM_STATUS_SIZE + 1];
		uint8_t buffer_accessories[SHAREMEM_ACCSIZE];
	
		/*機関車データ確認 */
		DATA_LOC gLocStatus[MAX_LOCSTATUS_WEB];

	
	public:
	
	
	
		ShareMemManager();
	
		void Init();
		void Clear();
		void Process();
		void WriteShareMemByte(word inPos, char inValue);
		void LOCMNG_UpdateAccBuffer(word inAccAddr, word inPosition);
		void LOCMNG_UpdateAcc(word inNo);
		boolean WriteSharedMem_CVReply(word inCVNo, uint8_t inValue);
		void RegisterShareMem(uint8_t inNumber, uint8_t inValue);
		void ClearShareMem();
		uint8_t *GetShareMemPointer();

	
		void LOCMNG_Clear(void);
		uint8_t LOCMNG_SetLocoSpeed(word inAddr, word inSpd);
		uint8_t LOCMNG_SetLocoDir(word inAddr, uint8_t inDir);
		uint8_t LOCMNG_SetLocoFunc(word inAddr, uint8_t inNo, uint8_t inPower);

		
};

#endif
