//////////////////////////////////////////////////////////////////////////////////
// 
// DSairLite UIManager
// Copyright(C)'2023- DesktopStation
// 
// 本ソフトウェアは、上記の著作者によって開発されたものであり、利用には
// ライセンス契約が必要となります。
// 
//////////////////////////////////////////////////////////////////////////////////
//

#include <arduino.h>
#include "hardware/adc.h" 
#include "OzOLED.h"
#include "UIManager.h"
#include "board_pins.h"
#include "DSVersion.h"


#define		OFFSET_DCCADDRESS		49152
#define		OFFSET_DCCACCADDRESS	14335
#define		SIZE_ACCSHIFT			6
#define		COUNT_GOTOHOME			10		//3秒

#define		UPDATEDISPLAY_INV_DIR	0b0000000000000001
#define		UPDATEDISPLAY_INV_FNC	0b0000000000000010
#define		UPDATEDISPLAY_INV_SPD	0b0000000000000100
#define		UPDATEDISPLAY_INV_PWR	0b0000000000001000
#define		UPDATEDISPLAY_INV_ACC	0b0000000000010000


bool GetDCCManufacture(word inID, char *opManufactureName, char *opCountryName);

//初期化する
UIManager::UIManager()
{
	
	//Pin Mode	
	gpio_init(PIN_OUT_MATBUTTON1);
	gpio_init(PIN_OUT_MATBUTTON2);
	gpio_init(PIN_OUT_MATBUTTON3);
	gpio_init(PIN_IN_BUTTON1);
	gpio_init(PIN_IN_BUTTON2);
	gpio_init(PIN_IN_BUTTON3);
	gpio_init(PIN_IN_BUTTON4);
	gpio_init(PIN_IN_BUTTON5);
	
	gpio_set_dir(PIN_OUT_MATBUTTON1, GPIO_OUT);
	gpio_set_dir(PIN_OUT_MATBUTTON2, GPIO_OUT);
	gpio_set_dir(PIN_OUT_MATBUTTON3, GPIO_OUT);
	gpio_set_dir(PIN_IN_BUTTON1, GPIO_IN);
	gpio_set_dir(PIN_IN_BUTTON2, GPIO_IN);
	gpio_set_dir(PIN_IN_BUTTON3, GPIO_IN);
	gpio_set_dir(PIN_IN_BUTTON4, GPIO_IN);
	gpio_set_dir(PIN_IN_BUTTON5, GPIO_IN);
	
	gpio_disable_pulls(PIN_IN_BUTTON1);
	gpio_disable_pulls(PIN_IN_BUTTON2);
	gpio_disable_pulls(PIN_IN_BUTTON3);
	gpio_disable_pulls(PIN_IN_BUTTON4);
	gpio_disable_pulls(PIN_IN_BUTTON5);
	
	/* Configuration of Analog */
	adc_init();
	adc_gpio_init(PIN_VR_SPEED);
	adc_gpio_init(PIN_EDC);
	adc_gpio_init(PIN_CURRENT);
	
	
	mode = UI_MODE_LOGO;
	mode_last = UI_MODE_LOGO;
	count_logo_off = COUNT_GOTOHOME;

	fnc_shift = 0;
	acc_shift = 0;
	memset(acc_data, 0, sizeof(acc_data));

	/*
	pSSID = NULL;
	pPASSWORD = NULL;
	pIPHost = NULL;
	LCD = NULL;
	callback_dscommand = NULL;
	*/

	//フラグ
	flags_update_display_inv = 0;
	
	//電流センサオフセット
	offset_current = 0;

	//VR検出用積分バッファ初期化
	buf_VR = 0;
	
	//WiFi状態
	WiFiStatus = 0;
	flag_wifi_client = 0;
	flag_wifi_client_last = 0;

	//screen
	number_screen_home = 0;
	cycle_anime = 0;
	cycle_anime_dir = 0;
	
	//エラー
	error_code = 0;
	
	Clear();
	
}

void UIManager::Clear()
{
	
	state = UI_STATE_IDLE;
	mode = UI_MODE_HOME;
	
	u32_button_data = 0;
	s32_speed_ref = 0;
	u32_button_data_last = 0;
	s32_speed_ref_last = 0;

	//マトリクスボタン
	gMButton_state = 0;
	gMButton_buttonrow[0] = 0;
	gMButton_buttonrow[1] = 0;
	gMButton_buttonrow[2] = 0;
	
	
	//デフォルト車両状態
	loco_address = 3;
	loco_speed = 0;
	loco_dir = 0;
	loco_func = 0;
	dcc_power = 0;
	
	//CV関連の初期化
	cv_address = 0;
	cv_value = 0;
	cv1_value = 0;
	cv17_value = 0;
	cv18_value = 0;
	cv29_value = 0;
	loco_address_edited = 0;
	
	//アナログ関連も
	pwmana_lightoffset = 0;
	pwmana_speed = 0;
	pwmana_dir = 1;
	pwmana_output = 0;
	
	//状態変数もクリア
	state_loco = 0;
	state_acc = 0;
	state_home = 0;
	state_cvr = 0;
	state_cvw = 0;
	state_addr = 0;
	state_cfg = 0;
	state_editaddr = 0;
	state_analog = 0;

	//車両情報の初期化
	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		locoDatas[i].mAddress = 0;
		locoDatas[i].mDir = 0;
		locoDatas[i].mFuncBuf = 0;
		locoDatas[i].mSpeed = 0;
	}
	


}

void UIManager::Init()
{
	

	/* OLED init */
	LCD.init();
	

	UpdateDisplay_inv_logo();
	
}


//---------------------------------------------------------------------------
// process
//---------------------------------------------------------------------------




void UIManager::ProcessInterval()
{

	if(state == UI_STATE_REFRESH)
	{
		return;
	}

	if(mode != UI_MODE_LOGO)
	{
		UpdateDisplay_inv_animation_power();
	}

	switch(mode)
	{
	case UI_MODE_LOGO:
		break;
	case UI_MODE_ACC:
	case UI_MODE_HOME:
	case UI_MODE_ANALOG:
	case UI_MODE_CVREAD:
	case UI_MODE_CVWRITE:
	case UI_MODE_LOCOCHK:
	case UI_MODE_LOCOEDIT:
		UpdateDisplay_inv_sensor_volt();
		break;

	case UI_MODE_LOCO:
		UpdateDisplay_inv_sensor_volt();
		UpdateDisplay_inv_sensor_amp();
		UpdateDisplay_inv_animation_dir();
		break;

	case UI_MODE_CONTROLINFO:
		if(cycle_anime == 0)
		{
			UpdateDisplay_inv_controllist();
		}		
		break;

	}

	//アニメーション用カウンタ
	if( dcc_power == 1 )
	{
		cycle_anime_dir++;

		if( cycle_anime_dir > 7)
		{
			cycle_anime_dir = 0;
		}		
	}
	else
	{
		cycle_anime_dir = 0;
	}



	cycle_anime++;

	if( cycle_anime > 3)
	{
		cycle_anime = 0;
	}

}


void UIManager::Process()
{

	u32_button_data_last = u32_button_data;
	u32_button_data = GetMatrixButton();
	
	//ダイヤルの検出
	s32_speed_ref_last = s32_speed_ref;
	s32_speed_ref = (GetVRSpeed() >> 2) & 0b1111111000;
	
	//RP2040のADCは有効精度が7bitくらいしかないので、下限を設ける
	if( s32_speed_ref < 8)
	{
		s32_speed_ref = 0;
	}

	if( s32_speed_ref >= 1016)
	{
		s32_speed_ref = 1023;
	}
	
	//DEBUG
/*
	if(s32_speed_ref_last != s32_speed_ref)
	{
		Serial.print("VR: ");
		Serial.println(s32_speed_ref);
	}

	if(u32_button_data_last != u32_button_data)
	{
		Serial.print("Buttons: ");
		Serial.println(u32_button_data);
	}
*/
	
	MonitorUserAction();
	
	
	switch(state)
	{
	case UI_STATE_IDLE:
		//何もしない


		//更新指令があったもの
		if( (flags_update_display_inv & UPDATEDISPLAY_INV_DIR) > 0)
		{
			UpdateDisplay_inv_dir();
		}

		if( (flags_update_display_inv & UPDATEDISPLAY_INV_FNC) > 0)
		{
			UpdateDisplay_inv_function();
		}

		if( (flags_update_display_inv & UPDATEDISPLAY_INV_SPD) > 0)
		{
			UpdateDisplay_inv_speed();
		}

		if( (flags_update_display_inv & UPDATEDISPLAY_INV_PWR) > 0)
		{
			UpdateDisplay_inv_power();
		}

		if( (flags_update_display_inv & UPDATEDISPLAY_INV_ACC) > 0)
		{
			UpdateDisplay_inv_accessory();
		}

		flags_update_display_inv = 0;


		break;
		
	case UI_STATE_REFRESH:
		//更新する
		UpdateDisplay();
		
		flags_update_display_inv = 0;

		//状態を戻す(頻繁に更新させない)
		state = UI_STATE_IDLE;
		break;
		
	}
	
	
	//起動時のロゴ表示を自動でオフする
	if(count_logo_off > 0)
	{
		count_logo_off--;
		
		if( count_logo_off == 0)
		{
			mode = UI_MODE_LOCO;
			state = UI_STATE_IDLE;
			cycle_anime = 1;
			UpdateDisplay();
		}
		
	}

	//スロットル存在チェック



	if( flag_wifi_client > 0)
	{
		flag_wifi_client--;

		if( flag_wifi_client_last == 0)
		{
			//表示する
			UpdateDisplay_inv_wifi();			
		}

		if( flag_wifi_client == 0)
		{
			//表示を消す
			UpdateDisplay_inv_wifi();
		}
	}

	flag_wifi_client_last = flag_wifi_client;	
	
}

void UIManager::DrawForm()
{

	/* 枠の生成 */
	for( uint8_t i = 0; i < 15; i++)
	{
		LCD.drawTOLine2(1, i,  0b00011000, 0xFF, false);
		LCD.drawTOLine2(7, i,  0b00011000, 0xFF, false);
	}
	
	for( uint8_t i = 0; i < 5; i++)
	{
		LCD.drawTOLine2( 2 + i, 0,  0xFF, 0b00011000, false);
		LCD.drawTOLine2( 2 + i, 15,  0xFF, 0b00011000, false);	
	}

	LCD.drawIcon(0, 1, 9, false);
	LCD.drawIcon(15, 1, 10, false);
	LCD.drawIcon(0, 7, 11, false);
	LCD.drawIcon(15, 7, 12, false);

}

void UIManager::UpdateDisplay_msgdlgr(const char *inText, int x, int y)
{
	
	LCD.clearDisplay();

	/* 枠の生成 */
	DrawForm();

	/* メッセージ表示 */
	LCD.printString("◇エラー！◇", 5, 1);
	LCD.printString(inText, x, y);
	LCD.printString("ENTER で閉じます", 4, 5);
	
}

void UIManager::UpdateDisplay_msgdlgInfo(const char *inText, int x, int y)
{
	
	LCD.clearDisplay();

	/* 枠の生成 */
	DrawForm();

	/* メッセージ表示 */
	LCD.printString("◇情報◇", 6, 1);
	LCD.printString(inText, x, y);
	LCD.printString("ENTER で閉じます", 4, 5);
	
}


void UIManager::UpdateDisplay_inv_power()
{
	
	if( dcc_power == 0)
	{
		LCD.printString("POWEROFF", 12, 0);
	}
	else
	{
		LCD.printString("POWER", 12, 0);
		LCD.drawIcon(15, 0, 1, false);
	}	
	
}

void UIManager::UpdateDisplay_inv_animation_power()
{
	
	if(mode == UI_MODE_ANALOG)
	{
		switch(cycle_anime)
		{
			case 0:
			case 2:
				LCD.printString("        ", 12, 0);
				break;
			case 1:
			case 3:
				LCD.printString(" PWMOUT ", 12, 0);
				break;
		}
	}
	else if( dcc_power == 0)
	{
		switch(cycle_anime)
		{
			case 0:
				LCD.printString("        ", 12, 0);
				break;
			case 2:
				LCD.printString("POWEROFF", 12, 0);
				break;
		}
	}
	else
	{
		
	}
	
}

void UIManager::UpdateDisplay_inv_dir()
{
	byte shift = 0;
	
	if( loco_dir <= 1)
	{
		LCD.drawIcon2(12, 1, 3, cycle_anime_dir, false);
		LCD.drawIcon2(15, 1, 3, cycle_anime_dir, false);
		LCD.printString("前進", 13, 1);
	}
	else
	{
		if( dcc_power == 1 )
		{
			shift = 7 - cycle_anime_dir;
		}
		else
		{
			shift = 0;
		}

		LCD.drawIcon2(12, 1, 2, shift, false);
		LCD.drawIcon2(15, 1, 2, shift, false);
		LCD.printString("後進", 13, 1);
	}
	
}

void UIManager::UpdateDisplay_inv_animation_dir()
{
	byte shift = 0;

	if( loco_dir <= 1)
	{
		LCD.drawIcon2(12, 1, 3, cycle_anime_dir, false);
		LCD.drawIcon2(15, 1, 3, cycle_anime_dir, false);
	}
	else
	{

		if( dcc_power == 1 )
		{
			shift = 7 - cycle_anime_dir;
		}
		else
		{
			shift = 0;
		}

		LCD.drawIcon2(12, 1, 2, shift, false);
		LCD.drawIcon2(15, 1, 2, shift, false);
	}
	
}

void UIManager::UpdateDisplay_inv_wifi()
{

	if( flag_wifi_client > 0)
	{
		LCD.printString("Wifi", 7, 0);
	}
	else
	{
		LCD.printString("    ", 7, 0);
	}


}

void UIManager::UpdateDisplay_inv_sensor_volt()
{
	char aVdc_char[5] = {0,0,0,0,0};
	int32_t aVdc_ad = GetVdc();

	//(1.2 / 11.2)
	int32_t aVdc_volt = aVdc_ad * 231 / (120 * 256);
	sprintf(aVdc_char, " %2dV", aVdc_volt);

	LCD.printString(aVdc_char, 9, 0);	

}

void UIManager::UpdateDisplay_inv_sensor_amp()
{

	char aIout_char[5] = {0,0,0,0,0};


	int32_t aIout_ad = GetCurrent();
	

	
	//2A流れた場合、20mV、47倍ゲインで0.94Vとなる。逆算すると、3.3V(4095)で7.0212A相当。
	int32_t aIout_amp = aIout_ad * 702 / 4095;
	aIout_amp /= 10;
	

	
	if( dcc_power == 0)
	{
		sprintf(aIout_char, " --A");
		
		//オフセット値代入
		offset_current = aIout_ad;
		
	}
	else
	{
		sprintf(aIout_char, "%1d.%1dA", aIout_amp / 10, aIout_amp % 10);
	}

	LCD.printString(aIout_char, 9, 1);
	
}

void UIManager::UpdateDisplay_inv_address(uint8_t inEditMode)
{
	
	char addr_buf[5];
	
	sprintf( addr_buf, "%04d", loco_address);
	
	if( inEditMode == 0)
	{
		LCD.printBigNumberX2(addr_buf, 0, 2);
	}
	else
	{
		LCD.printBigNumberX2NOT(addr_buf, 0, 2);
	}
	
}

void UIManager::UpdateDisplay_inv_acc_address(uint8_t inEditMode)
{
	
	char addr_buf[5];
	
	sprintf( addr_buf, "%04d", acc_address);
	
	if( inEditMode == 0)
	{
		LCD.printBigNumberX2(addr_buf, 0, 3);
	}
	else
	{
		LCD.printBigNumberX2NOT(addr_buf, 0, 3);
	}

	LCD.printString("表示するアクセサリアドレスを", 0, 1);
	LCD.printString("入力してください。(1-1024)", 0, 2);

	LCD.printString("ENTER:もどる", 9, 7);

	
}

void UIManager::UpdateDisplay_inv_function()
{
	char aBuf[4];
	uint8_t aFuncNo;
	uint8_t aX = 1;
	
	//LCD.printString("FN", 0, 7);
	
	for( int i = 0; i < 5; i++)
	{


		//上段
		aFuncNo = fnc_shift * 10 + i;
		
		if( aFuncNo > 31)
		{
			sprintf(aBuf,"   ");
			printString(aBuf, aX, 6);
		} 
		else if( (loco_func >> aFuncNo) & 1 > 0)
		{
			LCD.DrawFunctionIcon(aFuncNo, aX, 6, true);
		}
		else
		{
			LCD.DrawFunctionIcon(aFuncNo, aX, 6, false);
		}

		//下段
		aFuncNo = fnc_shift * 10 + 5 + i;
		
		if( aFuncNo > 31)
		{
			sprintf(aBuf,"   ");
			printString(aBuf, aX, 7);
		}
		else if( (loco_func >> aFuncNo) & 1 > 0)
		{
			LCD.DrawFunctionIcon(aFuncNo, aX, 7, true);
		}
		else
		{
			LCD.DrawFunctionIcon(aFuncNo, aX, 7, false);
		}		
		
		aX += 3;
	}
	
}


void UIManager::UpdateDisplay_inv_speed()
{
	
	uint32_t aSpeed = (loco_speed * 100) / 1023;
	
	if( 1023 <= ( ((loco_speed * 100) % 1023) * 2u) )
	{
		aSpeed++; //  四捨五入による切り上げ
	}
	
	char speed_char[10];
	String(aSpeed).toCharArray(speed_char, 4);
	
	LCD.printString("速度", 0, 0);
	LCD.printString("％", 5, 0);
	
	ConvertToZChar(aSpeed, 3, speed_char);
	LCD.printString(speed_char, 2, 0);

	LCD.drawSpeedBarW16(0, 1, loco_speed, false);
	
}

void UIManager::UpdateDisplay_inv_analog()
{
	
	uint32_t aSpeed = (loco_speed * 100) / 1023;
	
	if( 1023 <= ( ((loco_speed * 100) % 1023) * 2u) )
	{
		aSpeed++; //  四捨五入による切り上げ
	}
	
	char speed_char[10];
	String(aSpeed).toCharArray(speed_char, 4);
	
	LCD.printString("PWM 電圧", 3, 2);
	LCD.printString("％", 12, 2);
	
	ConvertToZChar(aSpeed, 3, speed_char);
	LCD.printString(speed_char, 9, 2);
	
	if( pwmana_dir == 1)
	{
		LCD.printString("FWD", 3, 6);
	}
	else
	{
		LCD.printString("REV", 3, 6);
	}
	
	//Speed Bar
	LCD.drawSpeedBar(4, 4, loco_speed, false);

	
	if( pwmana_lightoffset > 0)
	{
		LCD.printString("常点灯ON", 10, 6);
	}
	else
	{
		LCD.printString("NO LIGHT", 10, 6);
	}
}

void UIManager::UpdateDisplay_inv_logo()
{
	char aTemp[25];

	LCD.DrawLogo(1);
	
	LCD.printString("Copyright(C) DesktopStation", 1, 5);
	LCD.printString("Firmware Version", 4, 6);
	
	ConvertToZChar(FIRMWARE_VERSION, 8, aTemp);
	LCD.printString(aTemp, 4, 7);
}

uint8_t UIManager::GetAccessoryStatus(int32_t inAddress)
{
	uint8_t aBitShift = (inAddress % 8);
	
	return (acc_data[inAddress >> 3] >> aBitShift) & 1;
	
}

void UIManager::ConvertToZChar(int32_t inNumbers, int32_t inLength, char inData[])
{
	char aAddress[64];
	char aTemp;
	int32_t aPos = 0;
	
	switch(inLength)
	{
		case 1:
			sprintf(aAddress, "%1d", inNumbers);
			break;
		case 2:
			sprintf(aAddress, "%2d", inNumbers);
			break;
		case 3:
			sprintf(aAddress, "%3d", inNumbers);
			break;
		case 4:
			sprintf(aAddress, "%4d", inNumbers);
			break;
		case 5:
			sprintf(aAddress, "%5d", inNumbers);
			break;
		default:
			sprintf(aAddress, "%4d", inNumbers);
			break;
	}
	
	
	for( int i = 0; i < inLength; i++)
	{
		
		aTemp = 0;
		
		switch(aAddress[i])
		{
		case '1':
			aTemp = 0x91;
			break;
		case '2':
			aTemp = 0x92;
			break;
		case '3':
			aTemp = 0x93;
			break;
		case '4':
			aTemp = 0x94;
			break;
		case '5':
			aTemp = 0x95;
			break;
		case '6':
			aTemp = 0x96;
			break;
		case '7':
			aTemp = 0x97;
			break;
		case '8':
			aTemp = 0x98;
			break;
		case '9':
			aTemp = 0x99;
			break;
		case '0':
			aTemp = 0x90;
			break;
		default:
			aTemp = 0x00;
			break;
		}
		
		if( aTemp > 0)
		{
			//数値
			inData[aPos] = 0xEF;
			aPos++;
			inData[aPos] = 0xBC;
			aPos++;
			inData[aPos] = aTemp;
		}
		else
		{
			//全角スペース
			inData[aPos] = 0xE3;
			aPos++;
			inData[aPos] = 0x80;
			aPos++;
			inData[aPos] = 0x80;
		}
		
		aPos++;
		
	}
	
	inData[aPos] = 0x00;
	
}

void UIManager::UpdateDisplay_inv_accessory()
{
	
	int32_t aAccNo = (int32_t)acc_shift * SIZE_ACCSHIFT;
	char aAddress[13];
	
	//Serial.println(aAccNo);
	
	for( int i = 0; i < 3; i++)
	{
		if( GetAccessoryStatus(aAccNo + i) == 0)
		{
			LCD.drawBigNumber( i * 5 + 2, 2, 10);
			
		}
		else
		{
			LCD.drawBigNumber( i * 5 + 2, 2, 11);
		}
		
		sprintf(aAddress, "%4d", aAccNo + i + 1);
		LCD.printString(aAddress, i * 5 + 2, 4);
		
	}
	
	for( int i = 0; i < 3; i++)
	{
		if( GetAccessoryStatus(aAccNo + i + 3) == 0)
		{
			LCD.drawBigNumber( i * 5 + 2, 5, 10);
			
		}
		else
		{
			LCD.drawBigNumber( i * 5 + 2, 5, 11);
		}
		
		sprintf(aAddress, "%4d", aAccNo + i + 3 + 1);
		LCD.printString(aAddress, i * 5 + 2, 7);
	}

	//フッタに切り替え表示
	LCD.drawIcon(0, 6, 16, false);
	LCD.drawIcon(15, 6, 17, false);
	LCD.drawIcon(0, 7, 18, false);	
	LCD.drawIcon(15, 7, 19, false);
	
}

void UIManager::UpdateDisplay_inv_cvread(bool fin_read)
{
	char aBuf[13];

	LCD.printString("デコーダからCVを読みだします", 0, 1);
	LCD.printString("CV番号を設定しENTER でスタート", 0, 2);

	LCD.printString("ＣＶ番号", 0, 4);
	
	ConvertToZChar(cv_address, 4, aBuf);
	LCD.printStringNOT(aBuf, 5, 4);

	
	LCD.printString("ＣＶ値", 0, 6);
	
	if( fin_read == true)
	{
		if( cv_value == 0xFFFF)
		{
			//単純読み出しエラー
			LCD.printString("ERROR1", 5, 6);
		}
		else if ( (cv_value >= 0) && (cv_value <= 255))
		{
			ConvertToZChar(cv_value, 3, aBuf);
			LCD.printString(aBuf, 5, 6);
		}
		else
		{
			//不明なエラー
			LCD.printString("ERROR2", 5, 6);
		}
	}
	else
	{
		LCD.printString("------", 5, 6);
	}
	
	
}

void UIManager::UpdateDisplay_inv_cvwrite(bool fin_write)
{
	char aBuf[13];

	LCD.printString("デコーダにCVを書きだします", 0, 1);

	
	LCD.printString("ＣＶ番号", 0, 4);
	
	ConvertToZChar(cv_address, 4, aBuf);
	
	if( state_cvw == 0)
	{
		LCD.printStringNOT(aBuf, 5, 4);
		LCD.printString("CV番号を入力してください", 0, 2);	
	}
	else
	{
		LCD.printString(aBuf, 5, 4);
	}

	
	LCD.printString("ＣＶ値", 0, 6);
	
	ConvertToZChar(cv_value, 3, aBuf);
	
	if( state_cvw == 1)
	{
		LCD.printStringNOT(aBuf, 6, 6);
		LCD.printString("CV値を入力しENTER でスタート", 0, 2);	
	}
	else
	{
		LCD.printString(aBuf, 6, 6);
	}
	
}

void UIManager::UpdateDisplay_inv_addredit()
{
	char addr_buf[5];

	//メッセージ表示
	LCD.printString("1-9999のアドレスを入力ください", 0, 1);
	LCD.printString("Enter で書きこみスタート", 0, 2);
	
	//エラーなし
	//LCD.printString("アドレス", 4, 2);	


	
	sprintf( addr_buf, "%04d", loco_address_edited);
	
	LCD.printBigNumberX2NOT(addr_buf, 0, 3);


}

void UIManager::UpdateDisplay_inv_checkaddress()
{
	char aBuf[13];
	char aBuf_ManufactureName[36];
	char aBuf_CountryName[12];	
	word aAddress;
	
	if( (cv29_value == 0xFFFF) || (cv29_value == 0xFFFF) || (cv29_value == 0xFFFF))
	{
		//読み込みエラー
		UpdateDisplay_msgdlgr("アドレスチェック失敗", 3, 4);
	}
	else
	{
		
		
		LCD.clearDisplay();
		
		/* 枠の生成 */
		DrawForm();
		
		//メッセージ表示
		LCD.printString("◆探査結果◆", 5, 1);
		
		//エラーなし
		LCD.printString("アドレス", 4, 2);
		
		if( (cv29_value & 0b100000) == 0)
		{
			aAddress = cv1_value;
			LCD.printString("SHORT ADDRESS", 5, 3);
		}
		else
		{
			aAddress = cv18_value + ((cv17_value & 0b00111111) << 8);
			LCD.printString("LONG ADDRESS", 5, 3);
		}
		
		ConvertToZChar(aAddress, 4, aBuf);
		LCD.printString(aBuf, 8, 2);

		if( cv8_value != 0xFFFF)
		{
			//製造者IDを表示
			GetDCCManufacture(cv8_value, aBuf_ManufactureName, aBuf_CountryName);

			LCD.printString(aBuf_ManufactureName, (16 - (strlen(aBuf_ManufactureName) >> 1) ) / 2, 5);
			LCD.printString(aBuf_CountryName, (16 - (strlen(aBuf_CountryName) >> 1) ) / 2, 6);
		}
	}
	
}



void UIManager::UpdateDisplay_inv_controllist()
{
	char aBuf[32];
	uint8_t aCount = 0;

	for( int i = 0; i < 6; i++)
	{
		if( locoDatas[i].mAddress > 0 )
		{
			sprintf(aBuf, "%d ADDR:%4d %s SPD%3d", i, locoDatas[i].mAddress, locoDatas[i].mDir == 1 ? "FWD" : "REV", (locoDatas[i].mSpeed * 100) / 1023);

			LCD.printString(aBuf, 0, i + 1);

			aCount++;
		}
	}

	if( aCount == 0)
	{
		LCD.printString("操作中のロコはありません。", 0, 1);
	}


}

void UIManager::UpdateDisplay_inv_ModeMenu()
{
	switch(number_screen_home)
	{
	case 0:
		LCD.printBigNumber("1", 0, 1);
		LCD.printString("LOCO", 2, 1);
		LCD.printString("操作", 2, 2);

		LCD.printBigNumber("2", 5, 1);
		LCD.printString("進路", 7, 1);
		LCD.printString("操作", 7, 2);

		LCD.printBigNumber("3", 11, 1);
		LCD.printString("ＣＶ", 13, 1);
		LCD.printString("READ", 13, 2);

		LCD.printBigNumber("4", 0, 4);
		LCD.printString("ＣＶ", 2, 4);
		LCD.printString("WRITE", 2, 5);

		LCD.printBigNumber("5", 5, 4);
		LCD.printString("ANALOG", 7, 4);
		LCD.printString("運転", 7, 5);

		LCD.printBigNumber("6", 11, 4);
		LCD.printString("ADDR", 13, 4);
		LCD.printString("探査", 13, 5);

		//タブ表示
		LCD.drawIcon(0, 0, 14, false);
		LCD.printStringNOT("P1", 1, 0);
		LCD.drawIcon(2, 0, 13, false);
		LCD.drawIcon(3, 0, 15, false);
		LCD.printString("P2", 4, 0);
		//フッタ表示
		LCD.printString("1/2", 8, 7);			
		break;
	case 1:

		LCD.printBigNumber("1", 0, 1);
		LCD.printString("ADDR", 2, 1);
		LCD.printString("編集", 2, 2);

		LCD.printBigNumber("2", 5, 1);
		LCD.printString("CONFIG", 7, 1);
		LCD.printString("設定", 7, 2);

		LCD.printBigNumber("3", 11, 1);
		LCD.printString("STATUS", 13, 1);
		LCD.printString("情報", 13, 2);

		LCD.printBigNumber("4", 0, 4);
		LCD.printString("DEVICE", 2, 4);
		LCD.printString("INFO", 2, 5);
	
		LCD.printBigNumber("5", 5, 4);
		LCD.printString("WebApp", 7, 4);
		LCD.printString("QRCode", 7, 5);

		//タブ表示

		LCD.printString("P1", 1, 0);
		LCD.drawIcon(2, 0, 15, false);
		LCD.drawIcon(3, 0, 14, false);
		LCD.printStringNOT("P2", 4, 0);
		LCD.drawIcon(5, 0, 13, false);


		//フッタ表示
		LCD.printString("2/2", 8, 7);	

		break;
	}

	//ラインを引く
	for( int i = 0; i < 16; i++)
	{
		LCD.drawTOLine2(6, i,  0b00011000, 0xFF,false);
	}

	//フッタに切り替え表示
	LCD.printString("PAGE", 6, 7);	
	//LCD.printString("<<", 0, 7);
	LCD.drawIcon(0, 7, 18, false);

	//LCD.printString(">>", 15, 7);
	LCD.drawIcon(15, 7, 19, false);

	LCD.drawIcon(1, 7, 16, false);
	LCD.drawIcon(14, 7, 17, false);


}

void UIManager::UpdateDisplay()
{
	LCD.clearDisplay();
	
	UpdateDisplay_inv_power();
	UpdateDisplay_inv_wifi();	
	UpdateDisplay_inv_sensor_volt();
	
	switch(mode)
	{
	case UI_MODE_LOGO:
		
		break;
	case UI_MODE_HOME:
		//ホーム画面表示
		UpdateDisplay_inv_ModeMenu();
		
		break;
	case UI_MODE_LOCO:
		
		UpdateDisplay_inv_address();
		
		UpdateDisplay_inv_speed();
		UpdateDisplay_inv_function();
		UpdateDisplay_inv_dir();

		UpdateDisplay_inv_sensor_amp();
		break;
	case UI_MODE_ACC:
		
		LCD.printString("【進路操作】", 0, 0);
		
		switch(state_acc)
		{
			case 0:
				UpdateDisplay_inv_accessory();
				break;

			case 1:
				UpdateDisplay_inv_acc_address(1);
				break;
		}
		break;
	case UI_MODE_CVREAD:
		LCD.printString("【ＣＶREAD】", 0, 0);
		
		UpdateDisplay_inv_cvread(false);
		break;
	case UI_MODE_CVWRITE:
		LCD.printString("【ＣＶWRITE 】", 0, 0);
		
		UpdateDisplay_inv_cvwrite(false);

		break;
	case UI_MODE_ANALOG:
		LCD.printString("【アナログ】", 0, 0);
		UpdateDisplay_inv_analog();
		break;
	case UI_MODE_LOCOCHK:
		LCD.printString("【ADDR探査】", 0, 0);
		
		LCD.printString("ENTER でアドレス探査スタート", 1, 4);
		break;

	case UI_MODE_LOCOEDIT:
		LCD.printString("【ADDR編集】", 0, 0);

		UpdateDisplay_inv_addredit();
		break;

	case UI_MODE_DEVICEINFO:
		LCD.printString("【DEVICEINFO】", 0, 0);

		UpdateDisplay_inv_logo();
		break;

	case UI_MODE_CONTROLINFO:
		LCD.printString("【CTRL状態】", 0, 0);

		UpdateDisplay_inv_controllist();
		break;

	case UI_MODE_CONFIG:
		LCD.printString("【設定】", 0, 0);
		break;
		
	case UI_MODE_STATUS:
		LCD.printString("【状態】", 0, 0);
		
		//WiFiアドレス表示
		LCD.printString("WiFi:", 0, 1);
		
		if( WiFiStatus == 1)
		{
			LCD.printString("CONNECTED", 3, 1);
		}
		else
		{
			LCD.printString("No WiFi", 3, 1);
		}

		LCD.printString("Throttle:", 0, 2);
		
		if( flag_wifi_client > 0)
		{
			LCD.printString("Communicating", 5, 2);
		}
		else
		{
			LCD.printString("Not exist", 5, 2);
		}

		LCD.printString("SSID:", 0, 3);
		LCD.printString("PASS:", 0, 4);
		LCD.printString(pSSID, 3, 3);
		LCD.printString(pPASSWORD, 3, 4);

		
		break;

	case UI_MODE_QRURL:
		LCD.printString("【WebAppQR】", 0, 0);
		LCD.DrawQRCodeURL(8, 1);

		LCD.printString("右記QRコードを", 0, 2);
		LCD.printString("スマホ・タブで", 0, 3);
		LCD.printString("読取りください", 0, 4);
		LCD.printString("WebApp利用可能", 0, 5);
		LCD.printString("(192.168.42.1)", 0, 6);


		break;


	}

}

uint8_t UIManager::CheckButton(uint32_t inButton)
{
	uint8_t aButtonState = BUTTON_NONE;
	
	
	if( (u32_button_data_last & u32_button_data & inButton) > 0)
	{
		//押しっぱなし
		aButtonState = BUTTON_PRESSING;
	}
	else if( ((u32_button_data_last & inButton) == 0) && ((u32_button_data & inButton) > 0) )
	{
		//押しはじめ
		aButtonState = BUTTON_PRESS;
	}
	else if(((u32_button_data & inButton) == 0) && ((u32_button_data_last & inButton) > 0))
	{
		//離された
		aButtonState = BUTTON_LEAVED;
	}
	else
	{
		//押されていない
		
	}
	
	return aButtonState;
	
}

void UIManager::MonitorUserAction()
{
	/* モードに依存しない共通のボタン操作 */
	if( (CheckButton(UI_BUTTON_MODE) == BUTTON_PRESS) && (mode != UI_MODE_ERROR) && (mode != UI_MODE_HOME))
	{
		if( pwmana_output == 1)
		{
			//PWM出力を停止させる
			callback_dscommand("DC", 0, pwmana_dir, 0, 0);
			pwmana_output = 0;
		}
		
		//HOME画面へ
		mode = UI_MODE_HOME;
		state = UI_STATE_REFRESH;
	}
	
	if( (CheckButton(UI_BUTTON_POWER) == BUTTON_PRESS) && (mode != UI_MODE_ERROR) && (mode != UI_MODE_ANALOG) && (mode != UI_MODE_CVREAD) && (mode != UI_MODE_CVWRITE) && (mode != UI_MODE_LOCOCHK) && (mode != UI_MODE_LOCOEDIT))
	{
		callback_dscommand("PW", (dcc_power == 0) ? 1 : 0, 0, 0, 0);
		dcc_power = (dcc_power == 0) ? 1 : 0;
		
		//画面の一部だけ更新
		UpdateDisplay_inv_power();
	}
	
	/* エラー発生通知,画面自動切換 */
	if( error_code > 0)
	{
		
		
		switch(error_code)
		{
		default:
		case 0:
			break;
			
		case 2:
			UpdateDisplay_msgdlgr("低電圧エラー", 5, 3);
			break;
		case 4:
			UpdateDisplay_msgdlgr("過電圧エラー", 5, 3);
			break;
		case 8:
			UpdateDisplay_msgdlgr("過電流エラー", 5, 3);
			break;
		}
		
		callback_dscommand("PW",  0, 0, 0, 0);
		dcc_power = 0;
		
		mode_last = mode;
		mode = UI_MODE_ERROR;
		error_code = 0;
	}
	
	
	
	/* ダイヤルの監視 */
	if( ( s32_speed_ref_last != s32_speed_ref) && (mode != UI_MODE_ERROR))
	{
		loco_speed = s32_speed_ref;
		
		if( mode == UI_MODE_ANALOG)
		{
			pwmana_speed = loco_speed;
			
			//コマンド送信
			callback_dscommand("DC", (pwmana_speed < pwmana_lightoffset) ? pwmana_lightoffset : pwmana_speed, pwmana_dir, 0, 0);
			pwmana_output = 1;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_analog();
		
		}
		else
		{
			callback_dscommand("SP", loco_address + OFFSET_DCCADDRESS, loco_speed, 2, 0);
			
			if( mode == UI_MODE_LOCO)
			{
				//画面の一部だけ更新
				flags_update_display_inv |= UPDATEDISPLAY_INV_SPD;
			}
		}
		
		
		//Serial.print("Speed=");
		//Serial.println(loco_speed);
		

	}
	
	
	/* モード依存の操作 */
	switch(mode)
	{
	case UI_MODE_LOGO:
		
		break;
	case UI_MODE_HOME:
		//ホーム画面表示
		
		switch(number_screen_home)
		{
			case 0:
				if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
				{
					mode = UI_MODE_LOCO;
					state = UI_STATE_REFRESH;
					state_loco = 0;
				}
				
				if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
				{
					mode = UI_MODE_ACC;
					state = UI_STATE_REFRESH;
					state_acc = 0;
				}
				
				if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
				{
					mode = UI_MODE_CVREAD;
					state = UI_STATE_REFRESH;
					state_cvr = 0;
					cv_value = 0;
					cv_address = 0;
				}
				
				if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
				{
					mode = UI_MODE_CVWRITE;
					state = UI_STATE_REFRESH;
					state_cvw = 0;
					cv_value = 0;
					cv_address = 0;
				}
				
				if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
				{
					mode = UI_MODE_ANALOG;
					state = UI_STATE_REFRESH;
				}
				
				if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
				{
					mode = UI_MODE_LOCOCHK;
					state = UI_STATE_REFRESH;
				}
				break;

			case 1:

				if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
				{
					mode = UI_MODE_LOCOEDIT;
					state = UI_STATE_REFRESH;
				}

				if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
				{
					mode = UI_MODE_CONFIG;
					state = UI_STATE_REFRESH;
				}
				
				if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
				{
					mode = UI_MODE_STATUS;
					state = UI_STATE_REFRESH;
				}
				
				if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
				{
					mode = UI_MODE_DEVICEINFO;
					state = UI_STATE_REFRESH;
					state_cvw = 0;
				}

				if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
				{
					mode = UI_MODE_QRURL;
					state = UI_STATE_REFRESH;
				}

				break;


		}

		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			//ページ切り替え
			if( number_screen_home == 0)
			{
				number_screen_home = 1;
			}
			else
			{
				number_screen_home--;
			}

			state = UI_STATE_REFRESH;			
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			//ページ切り替え
			if( number_screen_home == 1)
			{
				number_screen_home = 0;
			}
			else
			{
				number_screen_home++;
			}

			state = UI_STATE_REFRESH;
		}
		
		
		break;
	case UI_MODE_LOCO:
		
		
		MonitorUserAction_LOCO();
		
		break;
	case UI_MODE_ACC:
		
		MonitorUserAction_ACC();
		
		break;
	case UI_MODE_CVREAD:
		
		//強制出力停止
		if( dcc_power == 1)
		{
			callback_dscommand("PW", 0, 0, 0, 0);
			dcc_power = 0;
		}
		
		MonitorUserAction_CVREAD();
		break;
	case UI_MODE_CVWRITE:
		
		//強制出力停止
		if( dcc_power == 1)
		{
			callback_dscommand("PW", 0, 0, 0, 0);
			dcc_power = 0;
		}
		
		MonitorUserAction_CVWRITE();
		break;
	case UI_MODE_ANALOG:
		
		//強制出力停止
		if( dcc_power == 1)
		{
			callback_dscommand("PW", 0, 0, 0, 0);
			dcc_power = 0;
		}
		
		MonitorUserAction_ANALOG();
		
		break;
	case UI_MODE_LOCOCHK:
		
		//強制出力停止
		if( dcc_power == 1)
		{
			callback_dscommand("PW", 0, 0, 0, 0);
			dcc_power = 0;
		}
		
		MonitorUserAction_ADDRREAD();
		break;


	case UI_MODE_LOCOEDIT:

		//強制出力停止
		if( dcc_power == 1)
		{
			callback_dscommand("PW", 0, 0, 0, 0);
			dcc_power = 0;
		}

		MonitorUserAction_EditLocoAddr();
		break;

	case UI_MODE_DEVICEINFO:

		break;

	case UI_MODE_CONFIG:
		break;
	case UI_MODE_STATUS:
		break;
	case UI_MODE_ERROR:
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			mode = mode_last;
			mode_last = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;			
		}
		
		break;

	case UI_MODE_QRURL:
		break;
	}

}

void UIManager::MonitorUserAction_LOCO()
{
	uint8_t aUpdated = 0;
	UI_LOCODATA aLocData;	

	
	if( CheckButton(UI_BUTTON_DIR) == BUTTON_PRESS)
	{
		callback_dscommand("DI", loco_address + OFFSET_DCCADDRESS,(loco_dir <= 1) ? 2 : 1, 0, 0);
		loco_dir = (loco_dir <= 1) ? 2 : 1;
		
		if( mode == UI_MODE_LOCO)
		{
			//画面の一部だけ更新
			UpdateDisplay_inv_dir();
		}
	}
	
	switch(state_loco)
	{
	case 0:
		//ファンクション操作モード（通常状態)
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			fnc_shift++;
			
			if( fnc_shift > 3)
			{
				fnc_shift = 0;
			}
			
			//画面の一部だけ更新
			UpdateDisplay_inv_function();
		}
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			//画面の一部だけ更新
			UpdateDisplay_inv_address(1);

			//ファンクション表示部分にメッセージ
			LCD.clearLine(6);
			LCD.clearLine(7);
			LCD.printString("1-9999のアドレスを入力ください", 0, 6);
			LCD.printString("ENTER でもどります", 0, 7);


			//いったん保存
			aLocData.mAddress = loco_address;
			aLocData.mFuncBuf = loco_func;
			aLocData.mDir = loco_dir;
			aLocData.mSpeed = loco_speed;

			StoreLocoData(aLocData);
			
			//アドレス変更モード
			state_loco = 1;
		}
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedFunction(0);
		}
		
		break;
	case 1:
		//アドレス変更モード
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			loco_address = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_address(1);
		}
		
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			if( loco_address == 0)
			{
				loco_address = 3;
			}
			else if( loco_address > 9999)
			{
				loco_address = 3;
			}
			else
			{
				//何もしない
			}

			//既存データの有無をサーチ
			aLocData.mAddress = loco_address;

			if( GetLocoData(&aLocData) == true)
			{
				//Serial.print("FOUND");
				//Serial.println(loco_address);
				loco_func = aLocData.mFuncBuf;
				loco_dir = aLocData.mDir;
				loco_speed = aLocData.mSpeed;
			}
			else
			{
				//Serial.print("NOT FOUND");
				//Serial.println(loco_address);
				
				loco_func = 0;
				loco_speed = 0;
				loco_dir = 0;
			}	

			StoreLocoData(aLocData);			
			
			//画面の一部だけ更新
			state = UI_STATE_REFRESH;
			/*
			LCD.clearLine(6);
			LCD.clearLine(7);
			UpdateDisplay_inv_address();
			UpdateDisplay_inv_function();
			UpdateDisplay_inv_dir();
			UpdateDisplay_inv_speed();
			*/
			
			//戻る
			state_loco = 0;
		}
		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			PressedAddress(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			PressedAddress(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			PressedAddress(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			PressedAddress(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			PressedAddress(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			PressedAddress(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			PressedAddress(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			PressedAddress(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			PressedAddress(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			PressedAddress(0);
		}		
		break;
		
		
	}
	
}

void UIManager::MonitorUserAction_EditLocoAddr()
{
	uint8_t aUpdated = 0;
	uint8_t aDecoderAddrType;
	uint8_t aTargetAddrType;
	
	switch(state_editaddr)
	{
	case 99:
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			state_editaddr = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;
			
		}
		break;
	case 1:
		//CV29判定

		aDecoderAddrType = (cv29_value & 0b100000) > 0 ? 1: 0;
		aTargetAddrType = (loco_address_edited > 127) ? 1 : 0;

		LCD.clearLine(1);
		LCD.clearLine(2);

		if( aDecoderAddrType != aTargetAddrType )
		{

			
			LCD.printString("CV29書きこみ中....", 0, 1);

			if( aTargetAddrType == 0 )
			{
				//SHORTにする
				cv29_value &= 0b11011111;
				LCD.printString("ショートアドレスに設定", 0, 2);
			}
			else
			{
				//LONGにする
				cv29_value |= 0b100000;
				LCD.printString("ロングアドレスに設定", 0, 2);
			}

			//CV29書き換え
			callback_dscommand("SV", loco_address + OFFSET_DCCADDRESS, 29, cv29_value, 0);

		}


		if( (cv29_value & 0b100000) == 0)
		{
			//SHORT ADDRESS
			state_editaddr = 10;
			LCD.clearLine(1);
			LCD.printString("CV1 書きこみ....", 0, 1);
		}
		else
		{
			//LONG ADDRESS
			state_editaddr = 2;
			LCD.clearLine(1);
			LCD.printString("CV17&CV18 書きこみ....", 0, 1);

		}
		
		break;

	case 2:
		//CV1 or CV17/18 Write
		//LONG ADDRESS
		cv17_value = 0b11000000 | (loco_address_edited >> 8);
		cv18_value = 0xFF & loco_address_edited;


		//CV書き出し開始
		callback_dscommand("SV", 3 + OFFSET_DCCADDRESS, 17, cv17_value, 0);

		state_editaddr = 3;		
		break;

	case 3:
		//CV1 or CV17/18 Write
		
		//CV書き出し開始
		callback_dscommand("SV", 3 + OFFSET_DCCADDRESS, 18, cv18_value, 0);	

		state_editaddr = 4;
		break;

	case 4:
		char aTempMsg[37];

		sprintf(aTempMsg, "アドレス%4dに設定しました", loco_address_edited);

		UpdateDisplay_msgdlgInfo(aTempMsg, 2, 3);
		state_editaddr = 99;
		break;

	case 10:
		//CV1
		cv_address = 1;
		cv_value = loco_address_edited & 0xFF;

		//CV書き出し開始
		callback_dscommand("SV", loco_address + OFFSET_DCCADDRESS, 1, cv_value, 0);		

		state_editaddr = 4;
		break;


	case 0:
		//アドレス変更モード
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			loco_address_edited = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_addredit();
		}
		
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			if( loco_address_edited == 0)
			{
				loco_address_edited = 3;
				UpdateDisplay_inv_addredit();
			}
			else if( loco_address_edited > 9999)
			{
				loco_address_edited = 3;
				UpdateDisplay_inv_addredit();
			}
			else
			{

				LCD.clearLine(1);
				LCD.clearLine(2);

				//書き込みを行う
				LCD.printString("CV29読みこみ中....", 0, 1);

				//CV読み出し開始
				callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 29, 0, 0);
				
				//値をセット
				cv29_value = cv_value;

				if( cv29_value == 0xFFFF )
				{
					UpdateDisplay_msgdlgr("CV29読みだし失敗", 3, 3);
					state_editaddr = 99;					
				}
				else
				{
					state_editaddr = 1;
				}
			}
		
		}
		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 1;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 2;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 3;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 4;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 5;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 6;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 7;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 8;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 9;
			//更新
			UpdateDisplay_inv_addredit();
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			loco_address_edited = loco_address_edited * 10 + 0;
			//更新
			UpdateDisplay_inv_addredit();
		}		
		break;
		
		
	}
	
}

void UIManager::MonitorUserAction_ACC()
{
	uint8_t aUpdated = 0;
	
	if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
	{
		acc_shift++;
		
		if( acc_shift > 169)
		{
			acc_shift = 0;
		}
		
		//画面更新
		UpdateDisplay_inv_accessory();
	}
	
	if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
	{
		acc_shift--;
		
		if( acc_shift < 0)
		{
			acc_shift = 169;
		}
		
		//画面更新
		UpdateDisplay_inv_accessory();
	}
	
	if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
	{
		acc_shift = 0;
		
		//画面更新
		UpdateDisplay_inv_accessory();
	}	
	
	switch(state_acc)
	{
	case 0:
		//ファンクション操作モード（通常状態)
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			//アドレス変更モード
			state_acc = 1;

			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;

		}
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			//ファンクション操作
			PressedAccessory(6);
		}
		
		break;
	case 1:
		//アドレス変更モード
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			//戻る
			state_acc = 0;

			if( acc_address > 0)
			{
				acc_shift = (acc_address - 1) / 6;
			}
			else
			{
				acc_shift = 0;
			}

			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;			
		}
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			acc_address = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_acc_address(1);
		}		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			PressedAddressAcc(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			PressedAddressAcc(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			PressedAddressAcc(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			PressedAddressAcc(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			PressedAddressAcc(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			PressedAddressAcc(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			PressedAddressAcc(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			PressedAddressAcc(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			PressedAddressAcc(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			PressedAddressAcc(0);
		}
		break;
		
		
	}
	
}

void UIManager::MonitorUserAction_CVREAD()
{
	uint8_t aUpdated = 0;

	
	switch(state_cvr)
	{
	case 99:
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			state_cvr = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;			
		}
		break;
	case 0:
		//CV番号設定モード
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			cv_address = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_cvread(false);
		}
		
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			if( (cv_address > 0) && (cv_address <= 1024))
			{
				//CV読み出し開始
				callback_dscommand("GV", loco_address + OFFSET_DCCADDRESS, cv_address, 0, 0);
				
				//戻る
				state_cvr = 0;
				
				//画面の一部だけ更新
				UpdateDisplay_inv_cvread(true);
				
			}
			else
			{
				UpdateDisplay_msgdlgr("！アドレス外です", 4, 3);
				state_cvr = 99;
				
			}
		}
		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			PressedCVNo(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			PressedCVNo(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			PressedCVNo(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			PressedCVNo(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			PressedCVNo(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			PressedCVNo(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			PressedCVNo(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			PressedCVNo(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			PressedCVNo(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			PressedCVNo(0);
		}		
		break;
		
		
	}
	
}

void UIManager::MonitorUserAction_CVWRITE()
{
	uint8_t aUpdated = 0;

	
	switch(state_cvw)
	{
	case 98:
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			state_cvw = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;			
		}
		break;
	case 99:
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			state_cvw = 1;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;
		}
		break;
	case 0:
		//CV番号設定モード
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			cv_address = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_cvwrite(false);
		}
		
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			if( (cv_address > 0) && (cv_address <= 1024))
			{
				//戻る
				state_cvw = 1;
				
				//画面の一部だけ更新
				UpdateDisplay_inv_cvwrite(false);
				
			}
			else
			{
				UpdateDisplay_msgdlgr("！アドレス外です", 4, 3);
				state_cvw = 98;
				
			}
		}
		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			PressedCVNoWrite(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			PressedCVNoWrite(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			PressedCVNoWrite(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			PressedCVNoWrite(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			PressedCVNoWrite(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			PressedCVNoWrite(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			PressedCVNoWrite(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			PressedCVNoWrite(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			PressedCVNoWrite(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			PressedCVNoWrite(0);
		}		
		break;

	case 1:
		//CV番号設定モード
		
		if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
		{
			
			cv_value = 0;
			
			//画面の一部だけ更新
			UpdateDisplay_inv_cvwrite(false);
		}
		
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			
			if( (cv_value >= 0) && (cv_value <= 255))
			{
				//CV書き出し開始
				callback_dscommand("SV", loco_address + OFFSET_DCCADDRESS, cv_address, cv_value, 0);
				
				UpdateDisplay_msgdlgInfo("CVを書きこみました。", 3, 3);
				state_cvw = 98;				
				
			}
			else
			{
				UpdateDisplay_msgdlgr("値は0-255 にしてください", 3, 3);
				state_cvw = 99;
				
			}
		}
		
		
		if( CheckButton(UI_BUTTON_1) == BUTTON_PRESS)
		{
			PressedCVNoWrite(1);
		}
		
		if( CheckButton(UI_BUTTON_2) == BUTTON_PRESS)
		{
			PressedCVNoWrite(2);
		}
		
		if( CheckButton(UI_BUTTON_3) == BUTTON_PRESS)
		{
			PressedCVNoWrite(3);
		}
		
		if( CheckButton(UI_BUTTON_4) == BUTTON_PRESS)
		{
			PressedCVNoWrite(4);
		}
		
		if( CheckButton(UI_BUTTON_5) == BUTTON_PRESS)
		{
			PressedCVNoWrite(5);
		}
		
		if( CheckButton(UI_BUTTON_6) == BUTTON_PRESS)
		{
			PressedCVNoWrite(6);
		}
		
		if( CheckButton(UI_BUTTON_7) == BUTTON_PRESS)
		{
			PressedCVNoWrite(7);
		}
		
		if( CheckButton(UI_BUTTON_8) == BUTTON_PRESS)
		{
			PressedCVNoWrite(8);
		}
		
		if( CheckButton(UI_BUTTON_9) == BUTTON_PRESS)
		{
			PressedCVNoWrite(9);
		}
		
		if( CheckButton(UI_BUTTON_0) == BUTTON_PRESS)
		{
			PressedCVNoWrite(0);
		}		
		break;
		
		
	}
	
}

void UIManager::MonitorUserAction_ADDRREAD()
{
	uint8_t aUpdated = 0;
	char aBuf[4];
	UI_LOCODATA aLocData;
	
	switch(state_addr)
	{
	case 99:
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			state_addr = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;
		}

		if( CheckButton(UI_BUTTON_DIR) == BUTTON_PRESS)
		{
			state_addr = 0;
			mode = UI_MODE_LOCO;
			state = UI_STATE_REFRESH;

			if( (cv29_value & 0b100000) == 0)
			{
				loco_address = cv1_value;
			}
			else
			{
				loco_address = cv18_value + ((cv17_value & 0b00111111) << 8);
			}

			//リミット調整
			if( loco_address == 0)
			{
				loco_address = 3;
			}
			else if( loco_address > 9999)
			{
				loco_address = 9999;
			}
			else
			{
				//何もしない
			}

			//既存データの有無をサーチ
			aLocData.mAddress = loco_address;

			if( GetLocoData(&aLocData) == true)
			{
				//Serial.print("FOUND");
				//Serial.println(loco_address);
				loco_func = aLocData.mFuncBuf;
				loco_dir = aLocData.mDir;
				loco_speed = aLocData.mSpeed;
			}
			else
			{
				//Serial.print("NOT FOUND");
				//Serial.println(loco_address);
				
				loco_func = 0;
				loco_speed = 0;
				loco_dir = 0;
			}	

			StoreLocoData(aLocData);			
		
			//戻る
			state_loco = 0;
			
			//画面の一部だけ更新
			//UpdateDisplay();
			state = UI_STATE_REFRESH;
		}		
		break;
	case 0:
		//CV番号設定モード
		
		if( CheckButton(UI_BUTTON_ENTER) == BUTTON_PRESS)
		{
			LCD.clearLine(4);
			LCD.printString("CV29読みだし中....", 1, 4);

			//CV読み出し開始
			callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 29, 0, 0);
			
			//値をセット
			cv29_value = cv_value;
			
			if( cv29_value == 0xFFFF )
			{
				UpdateDisplay_msgdlgr("CV29読みだし失敗", 4, 3);
				state_addr = 99;
				
			}
			else
			{
				LCD.printString("CV29の値を読み取りました。　", 1, 4);
				LCD.printString("CV29=", 1, 5);
				
				sprintf(aBuf, "%3d", cv29_value);
				LCD.printString(aBuf, 4, 5);
				
				//次へ
				state_addr = 1;		
			
			}
			
		}
		break;
	case 1:
			

		
		if( (cv29_value & 0b100000) == 0)
		{
			
			//CV1 読み出し開始
			callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 1, 0, 0);
			
			//値をセット
			cv1_value = cv_value;
			
			if( cv1_value == 0xFFFF )
			{
				UpdateDisplay_msgdlgr("CV1 読みだし失敗", 4, 3);
				state_addr = 99;
				
			}
			else
			{
				//製造者ID取得へ
				state_addr = 3;
			}
		}
		else
		{
			//CV17 読み出し開始
			callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 17, 0, 0);
			
			//値をセット
			cv17_value = cv_value;
			
			if( cv17_value == 0xFFFF )
			{
				UpdateDisplay_msgdlgr("CV17読みだし失敗", 4, 3);
				state_addr = 99;
				
			}
			else
			{
				LCD.printString("CV17の値を読み取りました", 1, 4);
				LCD.printString("CV17=", 1, 5);
				
				sprintf(aBuf, "%3d", cv17_value);
				LCD.printString(aBuf, 4, 5);
				
				//次へ
				state_addr = 2;
			}
		}
		break;
		
	case 2:
		
		//CV18 読み出し開始
		callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 18, 0, 0);
		
		//値をセット
		cv18_value = cv_value;
		
		if( cv18_value == 0xFFFF )
		{
			UpdateDisplay_msgdlgr("CV18読みだし失敗", 4, 3);
			state_addr = 99;
			
		}
		else
		{
			//製造者ID取得へ
			state_addr = 3;
		}
		
		break;

	case 3:

		//CV8 読み出し開始
		callback_dscommand("GV", 3 + OFFSET_DCCADDRESS, 8, 0, 0);
		
		//値をセット
		cv8_value = cv_value;
		
		//アドレスを表示
		UpdateDisplay_inv_checkaddress();
		
		//次へ
		state_addr = 99;
	break;

	}

}


void UIManager::MonitorUserAction_ANALOG()
{
	uint8_t aUpdated = 0;
	

	
	if( CheckButton(UI_BUTTON_DIR) == BUTTON_PRESS)
	{
		pwmana_dir = (pwmana_dir <= 1) ? 2 : 1;
		
		//コマンド送信
		callback_dscommand("DC", (pwmana_speed < pwmana_lightoffset) ? pwmana_lightoffset : pwmana_speed, pwmana_dir, 0, 0);
		pwmana_output = 1;
		
		//画面の一部だけ更新
		UpdateDisplay_inv_analog();
	}
	
	//ファンクション操作モード（通常状態)
	
	if( CheckButton(UI_BUTTON_FNC) == BUTTON_PRESS)
	{
		
		//5%分を常点灯として注入
		pwmana_lightoffset = (pwmana_lightoffset == 0) ? 51 : 0;
		
		//コマンド送信
		callback_dscommand("DC", (pwmana_speed < pwmana_lightoffset) ? pwmana_lightoffset : pwmana_speed, pwmana_dir, 0, 0);
		pwmana_output = 1;
		
		//画面の一部だけ更新
		UpdateDisplay_inv_analog();
	}
		
}

void UIManager::PressedFunction(int32_t inNo)
{
	
	uint32_t aFuncNo = fnc_shift * 10 + inNo;
	uint8_t aPower = 0;
	
	if( (loco_func >> aFuncNo) & 1 > 0)
	{
		//OFFにする
		loco_func &= ~(1 << aFuncNo);
		aPower = 0;
	}
	else
	{
		//ONにする
		loco_func |= (1 << aFuncNo);
		aPower = 1;
	}
	
	//ファンクション操作をコマンドで送信
	callback_dscommand("FN", loco_address + OFFSET_DCCADDRESS, aFuncNo, aPower, 0);
	
	//画面の一部だけ更新
	UpdateDisplay_inv_function();
	
}

void UIManager::PressedAccessory(int32_t inNo)
{
	
	uint32_t aAccAddress = acc_shift * SIZE_ACCSHIFT + inNo - 1;
	uint8_t aBitShift;
	uint8_t aCurrentPower;
	uint8_t aNextPower = 0;
	
	aBitShift = (aAccAddress % 8);
	
	aCurrentPower = (acc_data[aAccAddress >> 3] >> aBitShift) & 1;
	
	if( aCurrentPower == 0)
	{
		acc_data[aAccAddress >> 3] = acc_data[aAccAddress >> 3] | (1 << aBitShift);
		aNextPower = 1;
	}
	else
	{
		acc_data[aAccAddress >> 3] = acc_data[aAccAddress >> 3] & ~(1 << aBitShift);
		aNextPower = 0;
	}
	
	
	//ファンクション操作をコマンドで送信
	callback_dscommand("TO", aAccAddress + OFFSET_DCCACCADDRESS + 1, aNextPower, 0, 0);
	
	//画面の一部だけ更新
	UpdateDisplay_inv_accessory();
	
}

void UIManager::PressedCVNo(int32_t inNo)
{
	
	cv_address = cv_address * 10 + inNo;
	
	if( cv_address > 1024)
	{
		cv_address = 1024;
	}
	
	//画面の一部だけ更新
	UpdateDisplay_inv_cvread(false);
	
}

void UIManager::PressedCVNoWrite(int32_t inNo)
{
	
	switch(state_cvw)
	{
	
	case 1:
		cv_value = cv_value * 10 + inNo;
		
		if( cv_value > 255)
		{
			cv_value = 255;
		}
		
		//画面の一部だけ更新
		UpdateDisplay_inv_cvwrite(false);
		break;
		
	case 0:
		cv_address = cv_address * 10 + inNo;
		
		if( cv_address > 1024)
		{
			cv_address = 1024;
		}
		
		//画面の一部だけ更新
		UpdateDisplay_inv_cvwrite(false);
		break;
		
	}

	
}

void UIManager::PressedAddress(int32_t inNo)
{
	
	loco_address = loco_address * 10 + inNo;
	
	if( loco_address > 9999)
	{
		loco_address = loco_address % 10000;
	}
	
	//画面の一部だけ更新
	UpdateDisplay_inv_address(1);
	
}

void UIManager::PressedAddressAcc(int32_t inNo)
{
	
	acc_address = acc_address * 10 + inNo;
	
	if( acc_address > 1024)
	{
		acc_address = 1024;
	}
	
	//画面の一部だけ更新
	UpdateDisplay_inv_acc_address(1);
	
}
	

void UIManager::IntervalVRSpeed()
{
	
	uint32_t aSpeed = 0;
	
	for( int i = 0; i < 4; i++)
	{
		//ADCの選択(BEMFを選択しておく)
		adc_select_input(PIN_VR_SPEED_A0);
		aSpeed += adc_read();
	}

	aSpeed = aSpeed >> 2;

	//10Hz /2 = 5Hz LPF
	buf_VR = ((buf_VR * 128) + (aSpeed * 128)) >> 8;	


	//Serial.print("VR: ");
	//Serial.println(buf_VR);	
	
}

int32_t UIManager::GetVRSpeed()
{
	
	uint32_t aSpeed = buf_VR;
	
	//aSpeed &= 0x0FE0;//下位5bitは捨てる

	//Serial.print("VR: ");
	//Serial.println(buf_VR);	
	
	return (int32_t)aSpeed;
	
}




int32_t UIManager::GetVdc()
{
	
	//ADCの選択(BEMFを選択しておく)
	adc_select_input(PIN_EDC_A1);
	
	return (int32_t)adc_read();
	
}




int32_t UIManager::GetCurrent()
{
	
	//ADCの選択(BEMFを選択しておく)
	adc_select_input(PIN_CURRENT_A2);
	
	int32_t aIout_ad = (int32_t)adc_read();
	
	//出力中はオフセット結果を反映
	if(dcc_power == 1)
	{
		aIout_ad -= offset_current;
		
		if( aIout_ad < 0)
		{
			aIout_ad = 0;
		}
	}
	
	return aIout_ad;
	
}

void UIManager::InitMatrixButton()
{

	gMButton_state = 0;

	gMButton_buttonrow[0] = 0;
	gMButton_buttonrow[1] = 0;
	gMButton_buttonrow[2] = 0;

}

bool UIManager::IntervalMatrixButton()
{
	bool aRet = true;
	int i = 0;

	switch(gMButton_state)
	{
		case 0:
			gpio_put(PIN_OUT_MATBUTTON1, 1);
			gpio_put(PIN_OUT_MATBUTTON2, 0);
			gpio_put(PIN_OUT_MATBUTTON3, 0);
			break;
		case 1:
			gMButton_buttonrow[0] += gpio_get(PIN_IN_BUTTON1) << 0;
			gMButton_buttonrow[0] += gpio_get(PIN_IN_BUTTON2) << 1;
			gMButton_buttonrow[0] += gpio_get(PIN_IN_BUTTON3) << 2;
			gMButton_buttonrow[0] += gpio_get(PIN_IN_BUTTON4) << 3;
			gMButton_buttonrow[0] += gpio_get(PIN_IN_BUTTON5) << 4;

			gpio_put(PIN_OUT_MATBUTTON1, 0);
			gpio_put(PIN_OUT_MATBUTTON2, 1);
			gpio_put(PIN_OUT_MATBUTTON3, 0);
			break;
		case 2:
			gMButton_buttonrow[1] += gpio_get(PIN_IN_BUTTON1) << 0;
			gMButton_buttonrow[1] += gpio_get(PIN_IN_BUTTON2) << 1;
			gMButton_buttonrow[1] += gpio_get(PIN_IN_BUTTON3) << 2;
			gMButton_buttonrow[1] += gpio_get(PIN_IN_BUTTON4) << 3;
			gMButton_buttonrow[1] += gpio_get(PIN_IN_BUTTON5) << 4;

			gpio_put(PIN_OUT_MATBUTTON1, 0);
			gpio_put(PIN_OUT_MATBUTTON2, 0);
			gpio_put(PIN_OUT_MATBUTTON3, 1);
			break;
		case 3:
			gMButton_buttonrow[2] += gpio_get(PIN_IN_BUTTON1) << 0;
			gMButton_buttonrow[2] += gpio_get(PIN_IN_BUTTON2) << 1;
			gMButton_buttonrow[2] += gpio_get(PIN_IN_BUTTON3) << 2;
			gMButton_buttonrow[2] += gpio_get(PIN_IN_BUTTON4) << 3;
			gMButton_buttonrow[2] += gpio_get(PIN_IN_BUTTON5) << 4;

			gpio_put(PIN_OUT_MATBUTTON1, 0);
			gpio_put(PIN_OUT_MATBUTTON2, 0);
			gpio_put(PIN_OUT_MATBUTTON3, 0);
			aRet = false;
			break;	

	}

	gMButton_state++;

	return aRet;

}

uint32_t UIManager::GetMatrixButton()
{
	uint32_t aTemp_button = 0;
	
	//Analyze Buttons
	aTemp_button += gMButton_buttonrow[0] << 0;
	aTemp_button += gMButton_buttonrow[1] << 5;
	aTemp_button += gMButton_buttonrow[2] << 10;
	
	return aTemp_button;
}

void UIManager::SetResultCVValue(word inCVValue)
{
	cv_value = inCVValue;
}

void UIManager::NotifyWiFiClient()
{

	flag_wifi_client = 50;

}

void UIManager::NotifyError(word inErrorCode)
{
	error_code = inErrorCode;
}

void UIManager::NotifyDCCPower(word inPower)
{
	if( dcc_power != inPower)
	{
		dcc_power = inPower;
				
		flags_update_display_inv |= UPDATEDISPLAY_INV_PWR;
	}
}

void UIManager::NotifyDCCSpeed(word inAddress, word inSpeed)
{
	
	if( (loco_address + OFFSET_DCCADDRESS) == inAddress)
	{
		if( loco_speed != inSpeed)
		{
			loco_speed = inSpeed;
			
			if( mode == UI_MODE_LOCO)
			{
				flags_update_display_inv |= UPDATEDISPLAY_INV_SPD;
			}
		}
	}

	//内部車両データにも反映
	UpdateLocoDataSpeed(inAddress, inSpeed);
}

void UIManager::NotifyDCCDirection(word inAddress, word inDirection)
{
	if( (loco_address + OFFSET_DCCADDRESS) == inAddress)
	{
		if( loco_dir != inDirection)
		{
			loco_dir = inDirection;
			
			if( mode == UI_MODE_LOCO)
			{
				flags_update_display_inv |= UPDATEDISPLAY_INV_DIR;
			}
		}
	}

	//内部車両データにも反映
	UpdateLocoDataDirection(inAddress, inDirection);

}

void UIManager::NotifyDCCFunction(word inAddress, word inFuncNo, word inPower)
{
	if( (loco_address + OFFSET_DCCADDRESS) == inAddress)
	{
		
		if( ((loco_func >> inFuncNo) & 1) != inPower)
		{
			
			if( inPower == 0)
			{
				//OFFにする
				loco_func &= ~(1 << inFuncNo);
			}
			else
			{
				//ONにする
				loco_func |= (1 << inFuncNo);
			}
			
			if( mode == UI_MODE_LOCO)
			{
				flags_update_display_inv |= UPDATEDISPLAY_INV_FNC;
			}
		}
		else
		{
			//何もしない
		}
	}

	//内部車両データにも反映
	UpdateLocoDataFunction(inAddress, inFuncNo, inPower);

}

void UIManager::NotifyAccessory(int32_t inAddr, word inPower)
{
	
	uint32_t aAccAddress = inAddr;
	uint8_t aBitShift;
	uint8_t aCurrentPower;
	uint8_t aNextPower = 0;
	
	aBitShift = (aAccAddress % 8);
	
	aCurrentPower = (acc_data[aAccAddress >> 3] >> aBitShift) & 1;

	/*
	Serial.print("ACC");
	Serial.print(inAddr);
	Serial.print(",");
	Serial.print(aCurrentPower);
	Serial.print(",");
	Serial.println(inPower);
	*/

	if( inPower ==  aCurrentPower)
	{
		return;
	}
	
	if( aCurrentPower == 0)
	{
		acc_data[aAccAddress >> 3] = acc_data[aAccAddress >> 3] | (1 << aBitShift);
	}
	else
	{
		acc_data[aAccAddress >> 3] = acc_data[aAccAddress >> 3] & ~(1 << aBitShift);
	}
		

	if( (mode == UI_MODE_ACC) && ((aAccAddress / SIZE_ACCSHIFT) == acc_shift) )
	{
		//画面の一部だけ更新
		flags_update_display_inv |= UPDATEDISPLAY_INV_ACC;
	}	
}


void UIManager::StoreLocoData(UI_LOCODATA inLocData)
{
	int aIndex = -1;

	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		if( (locoDatas[i].mAddress == inLocData.mAddress) || (locoDatas[i].mAddress == 0))
		{
			aIndex = i;
			break;
		}
	}

	if( aIndex >= 0)
	{
		locoDatas[aIndex].mDir = inLocData.mDir;
		locoDatas[aIndex].mFuncBuf = inLocData.mFuncBuf;
		locoDatas[aIndex].mSpeed = inLocData.mSpeed;
		locoDatas[aIndex].mAddress = inLocData.mAddress;

	}
	else
	{
		/* 空き無し */
	}

}

bool UIManager::GetLocoData(UI_LOCODATA *opData)
{
	bool aRet = false;
	UI_LOCODATA aLocData;

	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		if( locoDatas[i].mAddress == (*opData).mAddress)
		{
			
			(*opData).mAddress = locoDatas[i].mAddress;
			(*opData).mSpeed = locoDatas[i].mSpeed;
			(*opData).mDir = locoDatas[i].mDir;
			(*opData).mFuncBuf = locoDatas[i].mFuncBuf;

			aRet = true;

			break;
		}

		if( locoDatas[i].mAddress == 0)
		{
			break;
		}
	}

	return aRet;
}

void UIManager::UpdateLocoDataSpeed(word inAddress, word inSpeed)
{

	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		if( locoDatas[i].mAddress == inAddress)
		{
			locoDatas[i].mSpeed = inSpeed;
			break;
		}

		if( locoDatas[i].mAddress == 0)
		{
			break;
		}

	}
}

void UIManager::UpdateLocoDataDirection(word inAddress, word inDir)
{

	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		if( locoDatas[i].mAddress == inAddress)
		{
			locoDatas[i].mDir = inDir;
			break;
		}

		if( locoDatas[i].mAddress == 0)
		{
			break;
		}

	}
}

void UIManager::UpdateLocoDataFunction(word inAddress, word inFuncNo, word inPower)
{
	int aIndex = -1;

	for( int i = 0; i < LEN_LOCODATA; i++)
	{
		if( locoDatas[i].mAddress == inAddress)
		{

			uint32_t aFunc = locoDatas[i].mFuncBuf;
			
			if( ((aFunc >> inFuncNo) & 1) != inPower)
			{
				
				if( inPower == 0)
				{
					//OFFにする
					aFunc &= ~(1 << inFuncNo);
				}
				else
				{
					//ONにする
					aFunc |= (1 << inFuncNo);
				}
			}
			else
			{
				//何もしない
			}

			locoDatas[i].mFuncBuf = aFunc;

			break;
		}

		if( locoDatas[i].mAddress == 0)
		{
			break;
		}

	}
}

/////////////////////////////////////////////
//
// LCD関連
//
/////////////////////////////////////////////

void UIManager::printString(char *s, int x, int y)
{
	LCD.printString(s, x, y);
}

void UIManager::printStringNOT(char *s, int x, int y)
{
	LCD.printStringNOT(s, x, y);
}

void UIManager::printStringNumWords(int a, int x, int y)
{

	if( a >= 10)
	{
		printStringNumWord( a % 10, x + 1, y);
		printStringNumWord( a / 10, x, y);
	}
	else
	{
		printStringNumWord( 99, x, y);			
		printStringNumWord( a, x + 1, y);		
	}
}

void UIManager::printStringNumWordsNOT(int a, int x, int y)
{

	if( a >= 10)
	{
		printStringNumWordNOT( a % 10, x + 1, y);
		printStringNumWordNOT( a / 10, x, y);
	}
	else
	{
		printStringNumWordNOT( 99, x, y);		
		printStringNumWordNOT( a, x + 1, y);		
	}
}

void UIManager::printStringNumWord(int a, int x, int y)
{
	char aBuf[3];

	switch( a)
	{
		case 0:
			sprintf(aBuf, "０");
			break;
		case 1:
			sprintf(aBuf, "１");
			break;
		case 2:
			sprintf(aBuf, "２");
			break;
		case 3:
			sprintf(aBuf, "３");
			break;
		case 4:
			sprintf(aBuf, "４");
			break;
		case 5:
			sprintf(aBuf, "５");
			break;
		case 6:
			sprintf(aBuf, "６");
			break;
		case 7:
			sprintf(aBuf, "７");
			break;
		case 8:
			sprintf(aBuf, "８");
			break;
		case 9:
			sprintf(aBuf, "９");
			break;
		default:
			sprintf(aBuf, "　");
			break;
	}

	LCD.printString(aBuf, x, y);
}

void UIManager::printStringNumWordNOT(int a, int x, int y)
{
	char aBuf[3];

	switch( a)
	{
		case 0:
			sprintf(aBuf, "０");
			break;
		case 1:
			sprintf(aBuf, "１");
			break;
		case 2:
			sprintf(aBuf, "２");
			break;
		case 3:
			sprintf(aBuf, "３");
			break;
		case 4:
			sprintf(aBuf, "４");
			break;
		case 5:
			sprintf(aBuf, "５");
			break;
		case 6:
			sprintf(aBuf, "６");
			break;
		case 7:
			sprintf(aBuf, "７");
			break;
		case 8:
			sprintf(aBuf, "８");
			break;
		case 9:
			sprintf(aBuf, "９");
			break;
		default:
			sprintf(aBuf, "　");
			break;
	}

	LCD.printStringNOT(aBuf, x, y);
}


void UIManager::drawSpeedBar(uint8_t x, uint8_t y, uint8_t speed, uint8_t inNot)
{
	LCD.drawSpeedBar(x, y, speed, false);
}

void UIManager::clearDisplay()
{
	LCD.lcd_clear();           //clear the screen and set start position to top left corner

}


bool GetDCCManufacture(word inID, char *opManufactureName, char *opCountryName)
{

	String aManufactureName = "";
	String aCountry = "";
	
	switch(inID)
	{
		case 1: aManufactureName = "CML Electronics"; aCountry = "United Kingdom";break;
		case 2: aManufactureName = "Train Technology"; aCountry = "Belgium";break;
		case 11: aManufactureName = "NCE Corporation"; aCountry = "United States";break;
		case 12: aManufactureName = "Wangrow Electronics"; aCountry = "United States";break;
		case 13: aManufactureName = "PublicDomain & DIY"; aCountry = "";break;
		case 14: aManufactureName = "PSI"; aCountry = "United States";break;
		case 15: aManufactureName = "Ramfixx(Wangrow)"; aCountry = "United States";break;
		case 17: aManufactureName = "AdvanceIC Engineering"; aCountry = "United States";break;
		case 18: aManufactureName = "JMRI"; aCountry = "United States";break;
		case 19: aManufactureName = "AMW"; aCountry = "Austria";break;
		case 20: aManufactureName = "T4T/Technology for Trains"; aCountry = "Germany";break;
		case 21: aManufactureName = "Kreischer Datentechnik"; aCountry = "Germany";break;
		case 22: aManufactureName = "KAM Industries"; aCountry = "United States";break;
		case 23: aManufactureName = "S Helper Service"; aCountry = "United States";break;
		case 24: aManufactureName = "MoBaTron.de"; aCountry = "Germany";break;
		case 25: aManufactureName = "Team Digital"; aCountry = "United States";break;
		case 26: aManufactureName = "MBTronik"; aCountry = "Germany";break;
		case 27: aManufactureName = "MTH Electric Trains"; aCountry = "United States";break;
		case 28: aManufactureName = "Heljan A/S"; aCountry = "Denmark";break;
		case 29: aManufactureName = "Mistral Train Models"; aCountry = "Belgium";break;
		case 30: aManufactureName = "Digsight"; aCountry = "China";break;
		case 31: aManufactureName = "Brelec"; aCountry = "Belgium";break;
		case 32: aManufactureName = "Regal Way Co. Ltd"; aCountry = "Hong Kong";break;
		case 33: aManufactureName = "Praecipuus"; aCountry = "Canada";break;
		case 34: aManufactureName = "Aristo-Craft Trains"; aCountry = "United States";break;
		case 35: aManufactureName = "Electronik&ModelProduktion"; aCountry = "Sweden";break;
		case 36: aManufactureName = "DCCconcepts"; aCountry = "Australia";break;
		case 37: aManufactureName = "NAC Services"; aCountry = "United States";break;
		case 38: aManufactureName = "Broadway Limited"; aCountry = "United States";break;
		case 39: aManufactureName = "Educational Computer "; aCountry = "United States";break;
		case 40: aManufactureName = "KATO Precision Models"; aCountry = "Japan";break;
		case 41: aManufactureName = "Passmann"; aCountry = "Germany";break;
		case 42: aManufactureName = "Digikeijs"; aCountry = "Netherlands";break;
		case 43: aManufactureName = "Ngineering"; aCountry = "United States";break;
		case 44: aManufactureName = "SPROG-DCC"; aCountry = "United Kingdom";break;
		case 45: aManufactureName = "ANE Model"; aCountry = "TAIWAN";break;
		case 46: aManufactureName = "GFB Designs"; aCountry = "United Kingdom";break;
		case 47: aManufactureName = "Capecom"; aCountry = "Australia";break;
		case 48: aManufactureName = "Hornby Hobbies"; aCountry = "United Kingdom";break;
		case 49: aManufactureName = "Joka Electronic"; aCountry = "Germany";break;
		case 50: aManufactureName = "N&Q Electronics"; aCountry = "Spain";break;
		case 51: aManufactureName = "DCC Supplies"; aCountry = "United Kingdom";break;
		case 52: aManufactureName = "Krois-Modell"; aCountry = "Austria";break;
		case 53: aManufactureName = "RautenhausDigitalVertrieb"; aCountry = "Germany";break;
		case 54: aManufactureName = "TCH Technology"; aCountry = "United States";break;
		case 55: aManufactureName = "QElectronics"; aCountry = "Germany";break;
		case 56: aManufactureName = "LDH"; aCountry = "Argentina";break;
		case 57: aManufactureName = "Rampino Elektronik"; aCountry = "Germany";break;
		case 58: aManufactureName = "KRES"; aCountry = "Germany";break;
		case 59: aManufactureName = "Tam Valley Depot"; aCountry = "United States";break;
		case 60: aManufactureName = "Bluecher-Electronic"; aCountry = "Germany";break;
		case 61: aManufactureName = "TrainModules"; aCountry = "Hungary";break;
		case 62: aManufactureName = "Tams Elektronik"; aCountry = "Germany";break;
		case 63: aManufactureName = "Noarail"; aCountry = "Australia";break;
		case 64: aManufactureName = "Digital Bahn"; aCountry = "Germany";break;
		case 65: aManufactureName = "Gaugemaster"; aCountry = "United Kingdom";break;
		case 66: aManufactureName = "Railnet Solutions"; aCountry = "United States";break;
		case 67: aManufactureName = "Heller Modenlbahn"; aCountry = "Germany";break;
		case 68: aManufactureName = "MAWE Elektronik"; aCountry = "Switzerland";break;
		case 69: aManufactureName = "E-Modell"; aCountry = "Germany";break;
		case 70: aManufactureName = "Rocrail"; aCountry = "Germany";break;
		case 71: aManufactureName = "New York Byano"; aCountry = "Hong Kong";break;
		case 72: aManufactureName = "MTB Model"; aCountry = "Czech Republic";break;
		case 73: aManufactureName = "The Electric Railroad"; aCountry = "United States";break;
		case 74: aManufactureName = "PpP Digital"; aCountry = "Spain";break;
		case 75: aManufactureName = "Digitools Elektronika"; aCountry = "Hungary";break;
		case 76: aManufactureName = "Auvidel"; aCountry = "Germany";break;
		case 77: aManufactureName = "LS Models Sprl"; aCountry = "Belgium";break;
		case 78: aManufactureName = "Tehnologistic (train-O-matic)"; aCountry = "Romania";break;
		case 79: aManufactureName = "Hattons Model Railways"; aCountry = "United Kingdom";break;
		case 80: aManufactureName = "Spectrum Engineering"; aCountry = "United States";break;
		case 81: aManufactureName = "GooVerModels"; aCountry = "Belgium";break;
		case 82: aManufactureName = "HAG Modelleisenbahn"; aCountry = "Switzerland";break;
		case 83: aManufactureName = "JSS-Elektronic"; aCountry = "Germany";break;
		case 84: aManufactureName = "Railflyer Model Prototypes"; aCountry = "Canada";break;
		case 85: aManufactureName = "Uhlenbrock"; aCountry = "Germany";break;
		case 86: aManufactureName = "Wekomm Engineering"; aCountry = "Germany";break;
		case 87: aManufactureName = "RR-Cirkits"; aCountry = "United States";break;
		case 88: aManufactureName = "HONS Model"; aCountry = "Hong Kong";break;
		case 89: aManufactureName = "Pojezdy.EU"; aCountry = "Czech Republic";break;
		case 90: aManufactureName = "Shourt Line"; aCountry = "United States";break;
		case 91: aManufactureName = "Railstars Limited"; aCountry = "United States";break;
		case 92: aManufactureName = "Tawcrafts"; aCountry = "United Kingdom";break;
		case 93: aManufactureName = "Kevtronics cc"; aCountry = "South Africa";break;
		case 94: aManufactureName = "Electroniscript  Inc."; aCountry = "United States";break;
		case 95: aManufactureName = "Sanda Kan Industrial"; aCountry = "Hong Kong";break;
		case 96: aManufactureName = "PRICOM Design"; aCountry = "United States";break;
		case 97: aManufactureName = "Doehler & Haas"; aCountry = "Germany";break;
		case 98: aManufactureName = "Harman DCC"; aCountry = "United Kingdom";break;
		case 99: aManufactureName = "Lenz Elektronik"; aCountry = "Germany";break;
		case 100: aManufactureName = "Trenes Digitales"; aCountry = "Argentina";break;
		case 101: aManufactureName = "Bachmann Trains"; aCountry = "United States";break;
		case 102: aManufactureName = "IntegratedSignalSystems"; aCountry = "United States";break;
		case 103: aManufactureName = "Nagasue System"; aCountry = "Japan";break;
		case 104: aManufactureName = "TrainTech"; aCountry = "Netherlands";break;
		case 105: aManufactureName = "Computer Dialysis France"; aCountry = "France";break;
		case 106: aManufactureName = "Opherline1"; aCountry = "France";break;
		case 107: aManufactureName = "Phoenix Sound Systems"; aCountry = "United States";break;
		case 108: aManufactureName = "Nagoden"; aCountry = "Japan";break;
		case 109: aManufactureName = "Viessmann"; aCountry = "Germany";break;
		case 110: aManufactureName = "AXJ Electronics"; aCountry = "China";break;
		case 111: aManufactureName = "Haber&Koenig(HKE)"; aCountry = "Austria";break;
		case 112: aManufactureName = "LSdigital"; aCountry = "Germany";break;
		case 113: aManufactureName = "QS Industries (QSI)"; aCountry = "United States";break;
		case 114: aManufactureName = "Benezan Electronics"; aCountry = "Spain";break;
		case 115: aManufactureName = "Dietz"; aCountry = "Germany";break;
		case 116: aManufactureName = "MyLocoSound"; aCountry = "Australia";break;
		case 117: aManufactureName = "cT Elektronik"; aCountry = "Austria";break;
		case 118: aManufactureName = "MUT"; aCountry = "Germany";break;
		case 119: aManufactureName = "W.S.Ataras Engineering"; aCountry = "United States";break;
		case 120: aManufactureName = "csikos-muhely"; aCountry = "Hungary";break;
		case 122: aManufactureName = "Berros"; aCountry = "Netherlands";break;
		case 123: aManufactureName = "Massoth Elektronik"; aCountry = "Germany";break;
		case 124: aManufactureName = "DCC-Gaspar-Electronic"; aCountry = "Hungary";break;
		case 125: aManufactureName = "ProfiLok"; aCountry = "Germany";break;
		case 126: aManufactureName = "Mollehem Gardsproduktion"; aCountry = "Sweden";break;
		case 127: aManufactureName = "Atlas Model Railroad"; aCountry = "United States";break;
		case 128: aManufactureName = "Frateschi Model Trains"; aCountry = "Brazil";break;
		case 129: aManufactureName = "Digitrax"; aCountry = "United States";break;
		case 130: aManufactureName = "cmOS Engineering"; aCountry = "Australia";break;
		case 131: aManufactureName = "Trix Modelleisenbahn"; aCountry = "Germany";break;
		case 132: aManufactureName = "ZTC"; aCountry = "United Kingdom";break;
		case 133: aManufactureName = "Intelligent Command Control"; aCountry = "United States";break;
		case 134: aManufactureName = "LaisDCC"; aCountry = "China";break;
		case 135: aManufactureName = "CVP Products"; aCountry = "United States";break;
		case 136: aManufactureName = "NYRS"; aCountry = "United States";break;
		case 138: aManufactureName = "Train ID Systems"; aCountry = "United States";break;
		case 139: aManufactureName = "RealRail Effects"; aCountry = "United States";break;
		case 140: aManufactureName = "Desktop Station"; aCountry = "Japan";break;
		case 141: aManufactureName = "Soundtraxx"; aCountry = "United States";break;
		case 142: aManufactureName = "SLOMO Railroad Models"; aCountry = "Japan";break;
		case 143: aManufactureName = "Model Rectifier Corp."; aCountry = "United States";break;
		case 144: aManufactureName = "DCC Train Automation"; aCountry = "United Kingdom";break;
		case 145: aManufactureName = "Zimo Elektronik"; aCountry = "Austria";break;
		case 146: aManufactureName = "Rails Europ Express"; aCountry = "France";break;
		case 147: aManufactureName = "Umelec Ing. Buero"; aCountry = "Switzerland";break;
		case 148: aManufactureName = "BLOCKsignalling"; aCountry = "United Kingdom";break;
		case 149: aManufactureName = "Rock Junction Controls"; aCountry = "United States";break;
		case 150: aManufactureName = "Wm. K. Walthers"; aCountry = "United States";break;
		case 151: aManufactureName = "ESU ElectronicSolutionsUlm"; aCountry = "Germany";break;
		case 152: aManufactureName = "Digi-CZ"; aCountry = "Czech Republic";break;
		case 153: aManufactureName = "Train Control Systems"; aCountry = "United States";break;
		case 154: aManufactureName = "Dapol Limited"; aCountry = "United Kingdom";break;
		case 155: aManufactureName = "Gebr. Fleischmann"; aCountry = "Germany";break;
		case 156: aManufactureName = "Nucky"; aCountry = "Japan";break;
		case 157: aManufactureName = "Kuehn"; aCountry = "Germany";break;
		case 158: aManufactureName = "Fucik"; aCountry = "Czech Republic";break;
		case 159: aManufactureName = "LGB"; aCountry = "Germany";break;
		case 160: aManufactureName = "MD Electronics"; aCountry = "Germany";break;
		case 161: aManufactureName = "Roco"; aCountry = "Austria";break;
		case 162: aManufactureName = "PIKO"; aCountry = "Germany";break;
		case 163: aManufactureName = "WP Railshops"; aCountry = "Canada";break;
		case 164: aManufactureName = "drM"; aCountry = "TAIWAN";break;
		case 165: aManufactureName = "Model Electronic Railway Group"; aCountry = "United Kingdom";break;
		case 166: aManufactureName = "Maison de DCC"; aCountry = "Japan";break;
		case 167: aManufactureName = "Helvest Systems"; aCountry = "Switzerland";break;
		case 168: aManufactureName = "Model Train Technology"; aCountry = "United States";break;
		case 169: aManufactureName = "AE Electronic"; aCountry = "China";break;
		case 170: aManufactureName = "AuroTrains"; aCountry = "United States";break;
		case 173: aManufactureName = "Arnold/Rivarossi"; aCountry = "Germany";break;
		case 186: aManufactureName = "BRAWA Modellspielwaren"; aCountry = "Germany";break;
		case 204: aManufactureName = "Con-Com"; aCountry = "Austria";break;
		case 225: aManufactureName = "Blue Digital"; aCountry = "Poland";break;
		case 238: aManufactureName = "NMRA Reserved"; aCountry = "United States";break;
		case 0xFFFF: aManufactureName = "ERROR"; aCountry = "Unknown";break;
	}
	

	
	if( aManufactureName.length() == 0)
	{
		return false;
	}
	else
	{

		aCountry = String(inID) + ", " + aCountry;

		aManufactureName.toCharArray(opManufactureName, aManufactureName.length() + 1);
		aCountry.toCharArray(opCountryName, aCountry.length() + 1);


		//Serial.println(opManufactureName);
		//Serial.println(opCountryName);		

		return true;
	}
}


void UIManager::RegisterWifi(char *inpSSID, char *inpPASS)
{

	pSSID = inpSSID;
	pPASSWORD = inpPASS;

}

